@@ -133,25 +133,26 @@ class PlainDumper:
 
     def __call__(self, d, value):
         try:
-            printer = self.printer.gen_printer(value)
+            printer = self.printer.gen_printer(value.nativeValue)
         except:
-            printer = self.printer.invoke(value)
+            printer = self.printer.invoke(value.nativeValue)
         lister = getattr(printer, 'children', None)
         children = [] if lister is None else list(lister())
-        d.putType(self.printer.name)
+        d.putType(value.nativeValue.type.name)
         val = printer.to_string()
         if isinstance(val, str):
             d.putValue(val)
         elif sys.version_info[0] <= 2 and isinstance(val, unicode):
             d.putValue(val)
         else: # Assuming LazyString
-            d.putCharArrayHelper(val.address, val.length, val.type)
+            d.putCharArrayValue(val.address, val.length,
+                                val.type.target().sizeof)
 
         d.putNumChild(len(children))
         if d.isExpanded():
             with Children(d):
                 for child in children:
-                    d.putSubItem(child[0], child[1])
+                    d.putSubItem(child[0], d.fromNativeValue(child[1]))
 
 def importPlainDumpers(args):
     if args == 'off':
@@ -245,17 +246,19 @@ class Dumper(DumperBase):
             #warn('TARGET TYPE: %s' % targetType)
             if targetType.code == gdb.TYPE_CODE_ARRAY:
                 val = self.Value(self)
-                val.laddress = toInteger(nativeValue.address)
-                val.nativeValue = nativeValue
             else:
-                # Cast may fail (e.g for arrays, see test for Bug5799)
-                val = self.fromNativeValue(nativeValue.cast(targetType))
-            val.type = self.fromNativeType(nativeType)
-            val.nativeValue = nativeValue
+                try:
+                    # Cast may fail for arrays, for typedefs to __uint128_t with
+                    # gdb.error: That operation is not available on integers
+                    # of more than 8 bytes.
+                    # See test for Bug5799, QTCREATORBUG-18450.
+                    val = self.fromNativeValue(nativeValue.cast(targetType))
+                except:
+                    val = self.Value(self)
             #warn('CREATED TYPEDEF: %s' % val)
-            return val
+        else:
+            val = self.Value(self)
 
-        val = self.Value(self)
         val.nativeValue = nativeValue
         if not nativeValue.address is None:
             val.laddress = toInteger(nativeValue.address)
@@ -354,10 +357,8 @@ class Dumper(DumperBase):
                 gdb.TYPE_CODE_STRING : TypeCodeFortranString,
             }[code]
             if tdata.code == TypeCodeEnum:
-                tdata.enumDisplay = lambda intval, addr : \
-                    self.nativeTypeEnumDisplay(nativeType, intval, 0)
-                tdata.enumHexDisplay = lambda intval, addr : \
-                    self.nativeTypeEnumDisplay(nativeType, intval, 1)
+                tdata.enumDisplay = lambda intval, addr, form : \
+                    self.nativeTypeEnumDisplay(nativeType, intval, form)
             if tdata.code == TypeCodeStruct:
                 tdata.lalignment = lambda : \
                     self.nativeStructAlignment(nativeType)
@@ -388,17 +389,13 @@ class Dumper(DumperBase):
         targs2 = self.listTemplateParametersManually(str(nativeType))
         return targs if len(targs) >= len(targs2) else targs2
 
-    def nativeTypeEnumDisplay(self, nativeType, intval, useHex):
-        if useHex:
-            format = lambda text, intval: '%s (0x%04x)' % (text, intval)
-        else:
-            format = lambda text, intval: '%s (%d)' % (text, intval)
+    def nativeTypeEnumDisplay(self, nativeType, intval, form):
         try:
             enumerators = []
             for field in nativeType.fields():
                 # If we found an exact match, return it immediately
                 if field.enumval == intval:
-                    return format(field.name, intval)
+                    return field.name + ' (' + (form % intval) + ')'
                 enumerators.append((field.name, field.enumval))
 
             # No match was found, try to return as flags
@@ -413,13 +410,11 @@ class Dumper(DumperBase):
                     found = True
             if not found or v != 0:
                 # Leftover value
-                flags.append('unknown:%d' % v)
-            return format(" | ".join(flags), intval)
+                flags.append('unknown: %d' % v)
+            return '(' + " | ".join(flags) + ') (' + (form % intval) + ')'
         except:
             pass
-        if useHex:
-            return '0x%04x' % intval;
-        return '%d' % intval
+        return form % intval
 
     def nativeTypeId(self, nativeType):
         if nativeType and (nativeType.code == gdb.TYPE_CODE_TYPEDEF):
@@ -715,14 +710,18 @@ class Dumper(DumperBase):
         self.reportResult(self.output)
 
     def parseAndEvaluate(self, exp):
+        val = self.nativeParseAndEvaluate(exp)
+        return None if val is None else self.fromNativeValue(val)
+
+    def nativeParseAndEvaluate(self, exp):
         #warn('EVALUATE "%s"' % exp)
         try:
             val = gdb.parse_and_eval(exp)
+            return val
         except RuntimeError as error:
             if self.passExceptions:
                 warn("Cannot evaluate '%s': %s" % (exp, error))
             return None
-        return self.fromNativeValue(val)
 
     def callHelper(self, rettype, value, function, args):
         # args is a tuple.
@@ -998,11 +997,24 @@ class Dumper(DumperBase):
         if self.isWindowsTarget():
             qtCoreMatch = re.match('.*Qt5?Core[^/.]*d?\.dll', name)
         else:
-            qtCoreMatch = re.match('.*/libQt5?Core[^/.]\.so', name)
+            qtCoreMatch = re.match('.*/libQt5?Core[^/.]*\.so', name)
 
         if qtCoreMatch is not None:
+            self.addDebugLibs(objfile)
             self.handleQtCoreLoaded(objfile)
 
+    def addDebugLibs(self, objfile):
+        # The directory where separate debug symbols are searched for
+        # is "/usr/lib/debug".
+        try:
+            cooked = gdb.execute('show debug-file-directory', to_string=True)
+            clean = cooked.split('"')[1]
+            newdir = '/'.join(objfile.filename.split('/')[:-1])
+            gdb.execute('set debug-file-directory %s:%s' % (clean, newdir))
+        except:
+            pass
+
+
     def handleQtCoreLoaded(self, objfile):
         fd, tmppath = tempfile.mkstemp()
         os.close(fd)
@@ -1010,7 +1022,12 @@ class Dumper(DumperBase):
         try:
             symbols = gdb.execute(cmd, to_string = True)
         except:
-            pass
+            # command syntax depends on gdb version - below is gdb 8+
+            cmd = 'maint print msymbols -objfile "%s" -- %s' % (objfile.filename, tmppath)
+            try:
+                symbols = gdb.execute(cmd, to_string = True)
+            except:
+                pass
         ns = ''
         with open(tmppath) as f:
             for line in f:
@@ -1338,7 +1355,7 @@ class Dumper(DumperBase):
 
             frame = frame.older()
             i += 1
-        self.reportResult('stack={frames=[' + self.output + '].report}')
+        self.reportResult('stack={frames=[' + self.output + ']}')
 
     def createResolvePendingBreakpointsHookBreakpoint(self, args):
         class Resolver(gdb.Breakpoint):
