@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -36,13 +31,18 @@
 #include <QHostAddress>
 #include <QRegExp>
 #include <QTimeZone>
+#include <QJsonArray>
+#include <QJsonDocument>
 
 #include <ctype.h>
 
+#include <utils/processhandle.h>
+
 #define QTC_ASSERT_STRINGIFY_HELPER(x) #x
 #define QTC_ASSERT_STRINGIFY(x) QTC_ASSERT_STRINGIFY_HELPER(x)
 #define QTC_ASSERT_STRING(cond) qDebug("SOFT ASSERT: \"" cond"\" in file " __FILE__ ", line " QTC_ASSERT_STRINGIFY(__LINE__))
 #define QTC_ASSERT(cond, action) if (cond) {} else { QTC_ASSERT_STRING(#cond); action; } do {} while (0)
+#define QTC_CHECK(cond) if (cond) {} else { QTC_ASSERT_STRING(#cond); } do {} while (0)
 
 namespace Debugger {
 namespace Internal {
@@ -58,31 +58,31 @@ uchar fromhex(uchar c)
     return UCHAR_MAX;
 }
 
-void skipCommas(const char *&from, const char *to)
+void skipCommas(const QChar *&from, const QChar *to)
 {
     while (*from == ',' && from != to)
         ++from;
 }
 
-void GdbMi::parseResultOrValue(const char *&from, const char *to)
+void GdbMi::parseResultOrValue(const QChar *&from, const QChar *to)
 {
-    while (from != to && isspace(*from))
+    while (from != to && isspace(from->unicode()))
         ++from;
 
-    //qDebug() << "parseResultOrValue: " << QByteArray(from, to - from);
+    //qDebug() << "parseResultOrValue: " << QString(from, to - from);
     parseValue(from, to);
     if (isValid()) {
-        //qDebug() << "no valid result in " << QByteArray(from, to - from);
+        //qDebug() << "no valid result in " << QString(from, to - from);
         return;
     }
     if (from == to || *from == '(')
         return;
-    const char *ptr = from;
-    while (ptr < to && *ptr != '=') {
+    const QChar *ptr = from;
+    while (ptr < to && *ptr != '=' && *ptr != ':') {
         //qDebug() << "adding" << QChar(*ptr) << "to name";
         ++ptr;
     }
-    m_name = QByteArray(from, ptr - from);
+    m_name = QString(from, ptr - from);
     from = ptr;
     if (from < to && *from == '=') {
         ++from;
@@ -90,21 +90,21 @@ void GdbMi::parseResultOrValue(const char *&from, const char *to)
     }
 }
 
-QByteArray GdbMi::parseCString(const char *&from, const char *to)
+QString GdbMi::parseCString(const QChar *&from, const QChar *to)
 {
-    QByteArray result;
-    //qDebug() << "parseCString: " << QByteArray(from, to - from);
+    QString result;
+    //qDebug() << "parseCString: " << QString(from, to - from);
     if (*from != '"') {
         qDebug() << "MI Parse Error, double quote expected";
         ++from; // So we don't hang
-        return QByteArray();
+        return QString();
     }
-    const char *ptr = from;
+    const QChar *ptr = from;
     ++ptr;
     while (ptr < to) {
         if (*ptr == '"') {
             ++ptr;
-            result = QByteArray(from + 1, ptr - from - 2);
+            result = QString(from + 1, ptr - from - 2);
             break;
         }
         if (*ptr == '\\') {
@@ -112,7 +112,7 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
             if (ptr == to) {
                 qDebug() << "MI Parse Error, unterminated backslash escape";
                 from = ptr; // So we don't hang
-                return QByteArray();
+                return QString();
             }
         }
         ++ptr;
@@ -121,11 +121,11 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
 
     int idx = result.indexOf('\\');
     if (idx >= 0) {
-        char *dst = result.data() + idx;
-        const char *src = dst + 1, *end = result.data() + result.length();
+        QChar *dst = result.data() + idx;
+        const QChar *src = dst + 1, *end = result.data() + result.length();
         do {
-            char c = *src++;
-            switch (c) {
+            QChar c = *src++;
+            switch (c.unicode()) {
                 case 'a': *dst++ = '\a'; break;
                 case 'b': *dst++ = '\b'; break;
                 case 'f': *dst++ = '\f'; break;
@@ -140,7 +140,7 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
                         int chars = 0;
                         uchar prod = 0;
                         while (true) {
-                            uchar val = fromhex(c);
+                            uchar val = fromhex(c.unicode());
                             if (val == UCHAR_MAX)
                                 break;
                             prod = prod * 16 + val;
@@ -150,7 +150,7 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
                         }
                         if (!chars) {
                             qDebug() << "MI Parse Error, unrecognized hex escape";
-                            return QByteArray();
+                            return QString();
                         }
                         *dst++ = prod;
                         break;
@@ -164,20 +164,20 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
                                 --src;
                                 break;
                             }
-                            prod = prod * 8 + c - '0';
+                            prod = prod * 8 + c.unicode() - '0';
                             if (++chars == 3 || src == end)
                                 break;
                             c = *src++;
                         }
                         if (!chars) {
                             qDebug() << "MI Parse Error, unrecognized backslash escape";
-                            return QByteArray();
+                            return QString();
                         }
                         *dst++ = prod;
                     }
             }
             while (src != end) {
-                char c = *src++;
+                QChar c = *src++;
                 if (c == '\\')
                     break;
                 *dst++ = c;
@@ -190,10 +190,10 @@ QByteArray GdbMi::parseCString(const char *&from, const char *to)
     return result;
 }
 
-void GdbMi::parseValue(const char *&from, const char *to)
+void GdbMi::parseValue(const QChar *&from, const QChar *to)
 {
-    //qDebug() << "parseValue: " << QByteArray(from, to - from);
-    switch (*from) {
+    //qDebug() << "parseValue: " << QString(from, to - from);
+    switch (from->unicode()) {
         case '{':
             parseTuple(from, to);
             break;
@@ -209,19 +209,18 @@ void GdbMi::parseValue(const char *&from, const char *to)
     }
 }
 
-
-void GdbMi::parseTuple(const char *&from, const char *to)
+void GdbMi::parseTuple(const QChar *&from, const QChar *to)
 {
-    //qDebug() << "parseTuple: " << QByteArray(from, to - from);
+    //qDebug() << "parseTuple: " << QString(from, to - from);
     //QTC_CHECK(*from == '{');
     ++from;
     parseTuple_helper(from, to);
 }
 
-void GdbMi::parseTuple_helper(const char *&from, const char *to)
+void GdbMi::parseTuple_helper(const QChar *&from, const QChar *to)
 {
     skipCommas(from, to);
-    //qDebug() << "parseTuple_helper: " << QByteArray(from, to - from);
+    //qDebug() << "parseTuple_helper: " << QString(from, to - from);
     m_type = Tuple;
     while (from < to) {
         if (*from == '}') {
@@ -238,9 +237,9 @@ void GdbMi::parseTuple_helper(const char *&from, const char *to)
     }
 }
 
-void GdbMi::parseList(const char *&from, const char *to)
+void GdbMi::parseList(const QChar *&from, const QChar *to)
 {
-    //qDebug() << "parseList: " << QByteArray(from, to - from);
+    //qDebug() << "parseList: " << QString(from, to - from);
     //QTC_CHECK(*from == '[');
     ++from;
     m_type = List;
@@ -252,37 +251,44 @@ void GdbMi::parseList(const char *&from, const char *to)
         }
         GdbMi child;
         child.parseResultOrValue(from, to);
-        if (child.isValid())
+        if (child.isValid()) {
             m_children.push_back(child);
-        skipCommas(from, to);
+            skipCommas(from, to);
+        } else {
+            ++from;
+        }
     }
 }
 
-static QByteArray ind(int indent)
+static QString ind(int indent)
 {
-    return QByteArray(2 * indent, ' ');
+    return QString(2 * indent, QChar(' '));
 }
 
-void GdbMi::dumpChildren(QByteArray * str, bool multiline, int indent) const
+void GdbMi::dumpChildren(QString * str, bool multiline, int indent) const
 {
-    for (size_t i = 0; i < m_children.size(); ++i) {
-        if (i != 0) {
+    bool first = true;
+    for (const GdbMi &child : *this) {
+        if (first) {
+            first = false;
+        } else {
             *str += ',';
             if (multiline)
                 *str += '\n';
         }
+
         if (multiline)
             *str += ind(indent);
-        *str += m_children.at(i).toString(multiline, indent);
+        *str += child.toString(multiline, indent);
     }
 }
 
-QByteArray GdbMi::escapeCString(const QByteArray &ba)
+QString GdbMi::escapeCString(const QString &ba)
 {
-    QByteArray ret;
+    QString ret;
     ret.reserve(ba.length() * 2);
     for (int i = 0; i < ba.length(); ++i) {
-        const uchar c = ba.at(i);
+        const ushort c = ba.at(i).unicode();
         switch (c) {
             case '\\': ret += "\\\\"; break;
             case '\a': ret += "\\a"; break;
@@ -307,9 +313,9 @@ QByteArray GdbMi::escapeCString(const QByteArray &ba)
     return ret;
 }
 
-QByteArray GdbMi::toString(bool multiline, int indent) const
+QString GdbMi::toString(bool multiline, int indent) const
 {
-    QByteArray result;
+    QString result;
     switch (m_type) {
         case Invalid:
             if (multiline)
@@ -352,37 +358,42 @@ QByteArray GdbMi::toString(bool multiline, int indent) const
     return result;
 }
 
-void GdbMi::fromString(const QByteArray &ba)
+void GdbMi::fromString(const QString &ba)
 {
-    const char *from = ba.constBegin();
-    const char *to = ba.constEnd();
+    const QChar *from = ba.constBegin();
+    const QChar *to = ba.constEnd();
     parseResultOrValue(from, to);
 }
 
-void GdbMi::fromStringMultiple(const QByteArray &ba)
+void GdbMi::fromStringMultiple(const QString &ba)
 {
-    const char *from = ba.constBegin();
-    const char *to = ba.constEnd();
+    const QChar *from = ba.constBegin();
+    const QChar *to = ba.constEnd();
     parseTuple_helper(from, to);
 }
 
 const GdbMi &GdbMi::operator[](const char *name) const
 {
     static GdbMi empty;
-    for (int i = 0, n = int(m_children.size()); i < n; ++i)
-        if (m_children.at(i).m_name == name)
-            return m_children.at(i);
+    for (const GdbMi &child : *this)
+        if (child.m_name == QLatin1String(name))
+            return child;
     return empty;
 }
 
 qulonglong GdbMi::toAddress() const
 {
-    QByteArray ba = m_data;
+    QString ba = m_data;
     if (ba.endsWith('L'))
         ba.chop(1);
     if (ba.startsWith('*') || ba.startsWith('@'))
         ba = ba.mid(1);
-    return ba.toULongLong(0, 0);
+    return ba.toULongLong(nullptr, 0);
+}
+
+Utils::ProcessHandle GdbMi::toProcessHandle() const
+{
+    return Utils::ProcessHandle(m_data.toULongLong());
 }
 
 //////////////////////////////////////////////////////////////////////////////////
@@ -391,23 +402,23 @@ qulonglong GdbMi::toAddress() const
 //
 //////////////////////////////////////////////////////////////////////////////////
 
-QByteArray DebuggerResponse::stringFromResultClass(ResultClass resultClass)
+QString DebuggerResponse::stringFromResultClass(ResultClass resultClass)
 {
     switch (resultClass) {
-        case ResultDone: return "done";
-        case ResultRunning: return "running";
-        case ResultConnected: return "connected";
-        case ResultError: return "error";
-        case ResultExit: return "exit";
-        default: return "unknown";
+        case ResultDone: return QLatin1String("done");
+        case ResultRunning: return QLatin1String("running");
+        case ResultConnected: return QLatin1String("connected");
+        case ResultError: return QLatin1String("error");
+        case ResultExit: return QLatin1String("exit");
+        default: return QLatin1String("unknown");
     }
 }
 
-QByteArray DebuggerResponse::toString() const
+QString DebuggerResponse::toString() const
 {
-    QByteArray result;
+    QString result;
     if (token != -1)
-        result = QByteArray::number(token);
+        result = QString::number(token);
     result += '^';
     result += stringFromResultClass(resultClass);
     if (data.isValid())
@@ -431,7 +442,8 @@ void extractGdbVersion(const QString &msg,
     const QChar dot(QLatin1Char('.'));
 
     const bool ignoreParenthesisContent = msg.contains(QLatin1String("rubenvb"))
-                                       || msg.contains(QLatin1String("openSUSE"));
+                                       || msg.contains(QLatin1String("openSUSE"))
+                                       || msg.contains(QLatin1String("SUSE Linux Enterprise"));
 
     const QChar parOpen(QLatin1Char('('));
     const QChar parClose(QLatin1Char(')'));
@@ -440,7 +452,7 @@ void extractGdbVersion(const QString &msg,
     QString build;
     bool inClean = true;
     bool inParenthesis = false;
-    foreach (QChar c, msg) {
+    for (QChar c : msg) {
         if (inClean && !cleaned.isEmpty() && c != dot && (c.isPunct() || c.isSpace()))
             inClean = false;
         if (ignoreParenthesisContent) {
@@ -485,12 +497,12 @@ void extractGdbVersion(const QString &msg,
 //
 //////////////////////////////////////////////////////////////////////////////////
 
-static QString quoteUnprintableLatin1(const QByteArray &ba)
+static QString quoteUnprintableLatin1(const QString &ba)
 {
     QString res;
     char buf[10];
     for (int i = 0, n = ba.size(); i != n; ++i) {
-        const unsigned char c = ba.at(i);
+        const unsigned char c = ba.at(i).unicode();
         if (isprint(c)) {
             res += QLatin1Char(c);
         } else {
@@ -512,7 +524,7 @@ static QTime timeFromData(int ms)
 }
 
 // Stolen and adapted from qdatetime.cpp
-static void getDateTime(qint64 msecs, int status, QDate *date, QTime *time)
+static void getDateTime(qint64 msecs, int status, QDate *date, QTime *time, int tiVersion)
 {
     enum {
         SECS_PER_DAY = 86400,
@@ -554,141 +566,125 @@ static void getDateTime(qint64 msecs, int status, QDate *date, QTime *time)
         ds = msecs;
     }
 
-    *date = (status & NullDate) ? QDate() : QDate::fromJulianDay(jd);
-    *time = (status & NullTime) ? QTime() : QTime::fromMSecsSinceStartOfDay(ds);
+    *date = ((status & NullDate) && tiVersion < 14) ? QDate() : QDate::fromJulianDay(jd);
+    *time = ((status & NullTime) && tiVersion < 14) ? QTime() : QTime::fromMSecsSinceStartOfDay(ds);
 }
 
-QString decodeData(const QByteArray &ba, int encoding)
+QString decodeData(const QString &ba, const QString &encoding)
 {
-    switch (encoding) {
-        case Unencoded8Bit: // 0
-            return quoteUnprintableLatin1(ba);
-        case Base64Encoded8BitWithQuotes: { // 1, used for QByteArray
-            const QChar doubleQuote(QLatin1Char('"'));
-            QString rc = doubleQuote;
-            rc += quoteUnprintableLatin1(QByteArray::fromBase64(ba));
-            rc += doubleQuote;
-            return rc;
+    if (encoding.isEmpty())
+        return quoteUnprintableLatin1(ba); // The common case.
+
+    if (encoding == "empty")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<empty>");
+    if (encoding == "minimumitemcount")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<at least %n items>", nullptr, ba.toInt());
+    if (encoding == "undefined")
+        return QLatin1String("Undefined");
+    if (encoding == "null")
+        return QLatin1String("Null");
+    if (encoding == "itemcount")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<%n items>", nullptr, ba.toInt());
+    if (encoding == "notaccessible")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<not accessible>");
+    if (encoding == "optimizedout")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<optimized out>");
+    if (encoding == "nullreference")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<null reference>");
+    if (encoding == "emptystructure")
+        return QLatin1String("{...}");
+    if (encoding == "uninitialized")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<uninitialized>");
+    if (encoding == "invalid")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<invalid>");
+    if (encoding == "notcallable")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<not callable>");
+    if (encoding == "outofscope")
+        return QCoreApplication::translate("Debugger::Internal::WatchHandler", "<out of scope>");
+
+    DebuggerEncoding enc(encoding);
+    QString result;
+    switch (enc.type) {
+        case DebuggerEncoding::Unencoded: {
+            result = quoteUnprintableLatin1(ba);
+            break;
         }
-        case Base64Encoded16BitWithQuotes: { // 2, used for QString
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromBase64(ba);
-            QString rc = doubleQuote;
-            rc += QString::fromUtf16(reinterpret_cast<const ushort *>
+        case DebuggerEncoding::HexEncodedLocal8Bit: {
+            const QByteArray decodedBa = QByteArray::fromHex(ba.toUtf8());
+            result = QString::fromLocal8Bit(decodedBa.data(), decodedBa.size());
+            break;
+        }
+        case DebuggerEncoding::HexEncodedLatin1: {
+            const QByteArray decodedBa = QByteArray::fromHex(ba.toUtf8());
+            result = QString::fromLatin1(decodedBa.data(), decodedBa.size());
+            break;
+        }
+        case DebuggerEncoding::HexEncodedUtf8: {
+            const QByteArray decodedBa = QByteArray::fromHex(ba.toUtf8());
+            result = QString::fromUtf8(decodedBa.data(), decodedBa.size());
+            break;
+        }
+        case DebuggerEncoding::HexEncodedUtf16: {
+            const QByteArray decodedBa = QByteArray::fromHex(ba.toUtf8());
+            result = QString::fromUtf16(reinterpret_cast<const ushort *>
                 (decodedBa.data()), decodedBa.size() / 2);
-            rc += doubleQuote;
-            return rc;
+            break;
         }
-        case Base64Encoded32BitWithQuotes: { // 3
-            const QByteArray decodedBa = QByteArray::fromBase64(ba);
-            const QChar doubleQuote(QLatin1Char('"'));
-            QString rc = doubleQuote;
-            rc += QString::fromUcs4(reinterpret_cast<const uint *>
+        case DebuggerEncoding::HexEncodedUcs4: {
+            const QByteArray decodedBa = QByteArray::fromHex(ba.toUtf8());
+            result = QString::fromUcs4(reinterpret_cast<const uint *>
                 (decodedBa.data()), decodedBa.size() / 4);
-            rc += doubleQuote;
-            return rc;
+            break;
         }
-        case Base64Encoded16Bit: { // 4, without quotes (see 2)
-            const QByteArray decodedBa = QByteArray::fromBase64(ba);
-            return QString::fromUtf16(reinterpret_cast<const ushort *>
-                (decodedBa.data()), decodedBa.size() / 2);
-        }
-        case Base64Encoded8Bit: { // 5, without quotes (see 1)
-            return quoteUnprintableLatin1(QByteArray::fromBase64(ba));
-        }
-        case Hex2EncodedLatin1WithQuotes: { // 6, %02x encoded 8 bit Latin1 data
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return doubleQuote + QString::fromLatin1(decodedBa, decodedBa.size()) + doubleQuote;
-        }
-        case Hex4EncodedLittleEndianWithQuotes: { // 7, %04x encoded 16 bit data
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return doubleQuote + QString::fromUtf16(reinterpret_cast<const ushort *>
-                (decodedBa.data()), decodedBa.size() / 2) + doubleQuote;
-        }
-        case Hex8EncodedLittleEndianWithQuotes: { // 8, %08x encoded 32 bit data
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return doubleQuote + QString::fromUcs4(reinterpret_cast<const uint *>
-                (decodedBa.data()), decodedBa.size() / 4) + doubleQuote;
-        }
-        case Hex2EncodedUtf8WithQuotes: { // 9, %02x encoded 8 bit UTF-8 data
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return doubleQuote + QString::fromUtf8(decodedBa) + doubleQuote;
-        }
-        case Hex8EncodedBigEndian: { // 10, %08x encoded 32 bit data
-            const QChar doubleQuote(QLatin1Char('"'));
-            QByteArray decodedBa = QByteArray::fromHex(ba);
-            for (int i = 0; i < decodedBa.size() - 3; i += 4) {
-                char c = decodedBa.at(i);
-                decodedBa[i] = decodedBa.at(i + 3);
-                decodedBa[i + 3] = c;
-                c = decodedBa.at(i + 1);
-                decodedBa[i + 1] = decodedBa.at(i + 2);
-                decodedBa[i + 2] = c;
-            }
-            return doubleQuote + QString::fromUcs4(reinterpret_cast<const uint *>
-                (decodedBa.data()), decodedBa.size() / 4) + doubleQuote;
-        }
-        case Hex4EncodedBigEndianWithQuotes: { // 11, %04x encoded 16 bit data
-            const QChar doubleQuote(QLatin1Char('"'));
-            QByteArray decodedBa = QByteArray::fromHex(ba);
-            for (int i = 0; i < decodedBa.size(); i += 2) {
-                char c = decodedBa.at(i);
-                decodedBa[i] = decodedBa.at(i + 1);
-                decodedBa[i + 1] = c;
-            }
-            return doubleQuote + QString::fromUtf16(reinterpret_cast<const ushort *>
-                (decodedBa.data()), decodedBa.size() / 2) + doubleQuote;
-        }
-        case Hex4EncodedLittleEndianWithoutQuotes: { // 12, see 7, without quotes
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return QString::fromUtf16(reinterpret_cast<const ushort *>
-                (decodedBa.data()), decodedBa.size() / 2);
-        }
-        case Hex2EncodedLocal8BitWithQuotes: { // 13, %02x encoded 8 bit UTF-8 data
-            const QChar doubleQuote(QLatin1Char('"'));
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return doubleQuote + QString::fromLocal8Bit(decodedBa) + doubleQuote;
-        }
-        case JulianDate: { // 14, an integer count
+        case DebuggerEncoding::JulianDate: {
             const QDate date = dateFromData(ba.toInt());
             return date.isValid() ? date.toString(Qt::TextDate) : QLatin1String("(invalid)");
         }
-        case MillisecondsSinceMidnight: {
+        case DebuggerEncoding::MillisecondsSinceMidnight: {
             const QTime time = timeFromData(ba.toInt());
             return time.isValid() ? time.toString(Qt::TextDate) : QLatin1String("(invalid)");
         }
-        case JulianDateAndMillisecondsSinceMidnight: {
+        case DebuggerEncoding::JulianDateAndMillisecondsSinceMidnight: {
             const int p = ba.indexOf('/');
             const QDate date = dateFromData(ba.left(p).toInt());
             const QTime time = timeFromData(ba.mid(p + 1 ).toInt());
             const QDateTime dateTime = QDateTime(date, time);
             return dateTime.isValid() ? dateTime.toString(Qt::TextDate) : QLatin1String("(invalid)");
         }
-        case IPv6AddressAndHexScopeId: { // 27, 16 hex-encoded bytes, "%" and the string-encoded scope
+        case DebuggerEncoding::HexEncodedUnsignedInteger:
+        case DebuggerEncoding::HexEncodedSignedInteger:
+            qDebug("not implemented"); // Only used in Arrays, see watchdata.cpp
+            return QString();
+        case DebuggerEncoding::HexEncodedFloat: {
+            const QByteArray s = QByteArray::fromHex(ba.toUtf8());
+            if (enc.size == 4) {
+                union { char c[4]; float f; } u = {{s[3], s[2], s[1], s[0]}};
+                return QString::number(u.f);
+            }
+            if (enc.size == 8) {
+                union { char c[8]; double d; } u = {{s[7], s[6], s[5], s[4], s[3], s[2], s[1], s[0]}};
+                return QString::number(u.d);
+            }
+            break;
+        }
+        case DebuggerEncoding::IPv6AddressAndHexScopeId: { // 16 hex-encoded bytes, "%" and the string-encoded scope
             const int p = ba.indexOf('%');
-            QHostAddress ip6(QString::fromLatin1(p == -1 ? ba : ba.left(p)));
+            QHostAddress ip6(p == -1 ? ba : ba.left(p));
             if (ip6.isNull())
                 break;
 
-            const QByteArray scopeId = p == -1 ? QByteArray() : QByteArray::fromHex(ba.mid(p + 1));
+            const QByteArray scopeId = p == -1 ? QByteArray() : QByteArray::fromHex(ba.mid(p + 1).toUtf8());
             if (!scopeId.isEmpty())
                 ip6.setScopeId(QString::fromUtf16(reinterpret_cast<const ushort *>(scopeId.constData()),
                                                   scopeId.length() / 2));
             return ip6.toString();
         }
-        case Hex2EncodedUtf8WithoutQuotes: { // 28, %02x encoded 8 bit UTF-8 data without quotes
-            const QByteArray decodedBa = QByteArray::fromHex(ba);
-            return QString::fromUtf8(decodedBa);
-        }
-        case DateTimeInternal: { // 29, DateTimeInternal: msecs, spec, offset, tz, status
+        case DebuggerEncoding::DateTimeInternal: { // DateTimeInternal: msecs, spec, offset, tz, status
             int p0 = ba.indexOf('/');
             int p1 = ba.indexOf('/', p0 + 1);
             int p2 = ba.indexOf('/', p1 + 1);
             int p3 = ba.indexOf('/', p2 + 1);
+            int p4 = ba.indexOf('/', p3 + 1);
 
             qint64 msecs = ba.left(p0).toLongLong();
             ++p0;
@@ -696,13 +692,15 @@ QString decodeData(const QByteArray &ba, int encoding)
             ++p1;
             qulonglong offset = ba.mid(p1, p2 - p1).toInt();
             ++p2;
-            QByteArray timeZoneId = QByteArray::fromHex(ba.mid(p2, p3 - p2));
+            QByteArray timeZoneId = QByteArray::fromHex(ba.mid(p2, p3 - p2).toUtf8());
             ++p3;
-            int status = ba.mid(p3).toInt();
+            int status = ba.mid(p3, p4 - p3).toInt();
+            ++p4;
+            int tiVersion = ba.mid(p4).toInt();
 
             QDate date;
             QTime time;
-            getDateTime(msecs, status, &date, &time);
+            getDateTime(msecs, status, &date, &time, tiVersion);
 
             QDateTime dateTime;
             if (spec == Qt::OffsetFromUTC) {
@@ -716,21 +714,15 @@ QString decodeData(const QByteArray &ba, int encoding)
             }
             return dateTime.toString();
         }
-        case Hex2EncodedFloat4: {
-            const QByteArray s = QByteArray::fromHex(ba);
-            QTC_ASSERT(s.size() == 4, break);
-            union { char c[4]; float f; } u = { { s[3], s[2], s[1], s[0] } };
-            return QString::number(u.f);
-        }
-        case Hex2EncodedFloat8: {
-            const QByteArray s = QByteArray::fromHex(ba);
-            QTC_ASSERT(s.size() == 8, break);
-            union { char c[8]; double d; } u = { { s[7], s[6], s[5], s[4], s[3], s[2], s[1], s[0] } };
-            return QString::number(u.d);
-        }
+        qDebug() << "ENCODING ERROR: " << enc.type;
+        return QCoreApplication::translate("Debugger", "<Encoding error>");
     }
-    qDebug() << "ENCODING ERROR: " << encoding;
-    return QCoreApplication::translate("Debugger", "<Encoding error>");
+
+    if (enc.quotes) {
+        const QChar doubleQuote(QLatin1Char('"'));
+        result = doubleQuote + result + doubleQuote;
+    }
+    return result;
 }
 
 //////////////////////////////////////////////////////////////////////////////////
@@ -739,88 +731,184 @@ QString decodeData(const QByteArray &ba, int encoding)
 //
 //////////////////////////////////////////////////////////////////////////////////
 
-void DebuggerCommand::argHelper(const char *name, const QByteArray &data)
+template<typename Value>
+QJsonValue addToJsonObject(const QJsonValue &args, const char *name, const Value &value)
 {
-    args.append('"');
-    args.append(name);
-    args.append("\":");
-    args.append(data);
-    args.append(",");
+    QTC_ASSERT(args.isObject() || args.isNull(), return args);
+    QJsonObject obj = args.toObject();
+    obj.insert(QLatin1String(name), value);
+    return obj;
 }
 
 void DebuggerCommand::arg(const char *name, int value)
 {
-    argHelper(name, QByteArray::number(value));
+    args = addToJsonObject(args, name, value);
 }
 
 void DebuggerCommand::arg(const char *name, qlonglong value)
 {
-    argHelper(name, QByteArray::number(value));
+    args = addToJsonObject(args, name, value);
 }
 
 void DebuggerCommand::arg(const char *name, qulonglong value)
 {
-    argHelper(name, QByteArray::number(value));
+    // gdb and lldb will correctly cast the value back to unsigned if needed, so this is no problem.
+    args = addToJsonObject(args, name, qint64(value));
 }
 
 void DebuggerCommand::arg(const char *name, const QString &value)
 {
-    arg(name, value.toUtf8().data());
-}
-
-void DebuggerCommand::arg(const char *name, const QByteArray &value)
-{
-    arg(name, value.data());
+    args = addToJsonObject(args, name, value);
 }
 
 void DebuggerCommand::arg(const char *name, const char *value)
 {
-    args.append('"');
-    args.append(name);
-    args.append("\":\"");
-    args.append(value);
-    args.append("\",");
+    args = addToJsonObject(args, name, QLatin1String(value));
+}
+
+void DebuggerCommand::arg(const char *name, const QList<int> &list)
+{
+    QJsonArray numbers;
+    for (int item : list)
+        numbers.append(item);
+    args = addToJsonObject(args, name, numbers);
+}
+
+void DebuggerCommand::arg(const char *name, const QStringList &list)
+{
+    QJsonArray arr;
+    for (const QString &item : list)
+        arr.append(toHex(item));
+    args = addToJsonObject(args, name, arr);
 }
 
 void DebuggerCommand::arg(const char *value)
 {
-    args.append("\"");
-    args.append(value);
-    args.append("\",");
+    QTC_ASSERT(args.isArray() || args.isNull(), return);
+    QJsonArray arr = args.toArray();
+    arr.append(QLatin1String(value));
+    args = arr;
 }
 
-void DebuggerCommand::beginList(const char *name)
+void DebuggerCommand::arg(const char *name, bool value)
 {
-    if (name) {
-        args += '"';
-        args += name;
-        args += "\":";
+    args = addToJsonObject(args, name, value);
+}
+
+void DebuggerCommand::arg(const char *name, const QJsonValue &value)
+{
+    args = addToJsonObject(args, name, value);
+}
+
+static QJsonValue translateJsonToPython(const QJsonValue &value)
+{
+    // TODO: Verify that this covers all incompatibilities between python and json,
+    //       e.g. number format and precision
+
+    switch (value.type()) {
+    // Undefined is not a problem as the JSON generator ignores that.
+    case QJsonValue::Null:
+        // Python doesn't understand "null"
+        return QJsonValue(0);
+    case QJsonValue::Bool:
+        // Python doesn't understand lowercase "true" or "false"
+        return QJsonValue(value.toBool() ? 1 : 0);
+    case QJsonValue::Object: {
+        QJsonObject object = value.toObject();
+        for (QJsonObject::iterator i = object.begin(); i != object.end(); ++i)
+            i.value() = translateJsonToPython(i.value());
+        return object;
     }
-    args += '[';
-}
-
-void DebuggerCommand::endList()
-{
-    if (args.endsWith(','))
-        args.chop(1);
-    args += "],";
-}
-
-void DebuggerCommand::beginGroup(const char *name)
-{
-    if (name) {
-        args += '"';
-        args += name;
-        args += "\":";
+    case QJsonValue::Array: {
+        QJsonArray array = value.toArray();
+        for (QJsonArray::iterator i = array.begin(); i != array.end(); ++i)
+            *i = translateJsonToPython(*i);
+        return array;
+    }
+    default:
+        return value;
     }
-    args += '{';
 }
 
-void DebuggerCommand::endGroup()
+QString DebuggerCommand::argsToPython() const
 {
-    if (args.endsWith(','))
-        args.chop(1);
-    args += "},";
+    QJsonValue pythonCompatible(translateJsonToPython(args));
+    if (pythonCompatible.isArray())
+        return QString::fromUtf8(QJsonDocument(pythonCompatible.toArray()).toJson(QJsonDocument::Compact));
+    else
+        return QString::fromUtf8(QJsonDocument(pythonCompatible.toObject()).toJson(QJsonDocument::Compact));
+}
+
+QString DebuggerCommand::argsToString() const
+{
+    return args.toString();
+}
+
+DebuggerEncoding::DebuggerEncoding(const QString &data)
+{
+    const QVector<QStringRef> l = data.splitRef(QLatin1Char(':'));
+
+    const QStringRef &t = l.at(0);
+    if (t == "latin1") {
+        type = HexEncodedLatin1;
+        size = 1;
+        quotes = true;
+    } else if (t == "local8bit") {
+        type = HexEncodedLocal8Bit;
+        size = 1;
+        quotes = true;
+    } else if (t == "utf8") {
+        type = HexEncodedUtf8;
+        size = 1;
+        quotes = true;
+    } else if (t == "utf16") {
+        type = HexEncodedUtf16;
+        size = 2;
+        quotes = true;
+    } else if (t == "ucs4") {
+        type = HexEncodedUcs4;
+        size = 4;
+        quotes = true;
+    } else if (t == "int") {
+        type = HexEncodedSignedInteger;
+    } else if (t == "uint") {
+        type = HexEncodedUnsignedInteger;
+    } else if (t == "float") {
+        type = HexEncodedFloat;
+    } else if (t == "juliandate") {
+        type = JulianDate;
+    } else if (t == "juliandateandmillisecondssincemidnight") {
+        type = JulianDateAndMillisecondsSinceMidnight;
+    } else if (t == "millisecondssincemidnight") {
+        type = MillisecondsSinceMidnight;
+    } else if (t == "ipv6addressandhexscopeid") {
+        type = IPv6AddressAndHexScopeId;
+    } else if (t == "datetimeinternal") {
+        type = DateTimeInternal;
+    } else if (!t.isEmpty()) {
+        qDebug() << "CANNOT DECODE ENCODING" << data;
+    }
+
+    if (l.size() >= 2)
+        size = l.at(1).toInt();
+
+    if (l.size() >= 3)
+        quotes = bool(l.at(2).toInt());
+}
+
+QString DebuggerEncoding::toString() const
+{
+    return QString("%1:%2:%3").arg(type).arg(size).arg(quotes);
+}
+
+QString fromHex(const QString &str)
+{
+    return QString::fromUtf8(QByteArray::fromHex(str.toUtf8()));
+}
+
+QString toHex(const QString &str)
+{
+    return QString::fromUtf8(str.toUtf8().toHex());
 }
 
 } // namespace Internal
