@@ -72,7 +72,7 @@ void addItemToStackedContainer(const SelectionContext &selectionContext);
 void increaseIndexOfStackedContainer(const SelectionContext &selectionContext);
 void decreaseIndexOfStackedContainer(const SelectionContext &selectionContext);
 void addTabBarToStackedContainer(const SelectionContext &selectionContext);
-bool addImageToProject(const QString &fileName, const QString &directory);
+bool addImageToProject(const QStringList &fileNames, const QString &directory);
 
 } // namespace ModelNodeOperationso
 } //QmlDesigner
