@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -34,9 +30,18 @@
 #include "internalnode_p.h"
 #include "nodeinstanceview.h"
 #include <qmlstate.h>
+#include <qmltimeline.h>
 
+#ifndef QMLDESIGNER_TEST
+#include <qmldesignerplugin.h>
+#include <viewmanager.h>
+#endif
+
+#include <coreplugin/helpmanager.h>
 #include <utils/qtcassert.h>
 
+#include <QRegExp>
+
 namespace QmlDesigner {
 
 
@@ -63,7 +68,7 @@ AbstractView::~AbstractView()
 */
 void AbstractView::setModel(Model *model)
 {
-    Q_ASSERT(model != 0);
+    Q_ASSERT(model != nullptr);
     if (model == m_model.data())
         return;
 
@@ -108,7 +113,7 @@ const ModelNode AbstractView::rootModelNode() const
 ModelNode AbstractView::rootModelNode()
 {
     Q_ASSERT(model());
-    return  ModelNode(model()->d->rootNode(), model(), this);
+    return ModelNode(model()->d->rootNode(), model(), this);
 }
 
 /*!
@@ -125,7 +130,8 @@ WidgetInfo AbstractView::createWidgetInfo(QWidget *widget,
                                           const QString &uniqueId,
                                           WidgetInfo::PlacementHint placementHint,
                                           int placementPriority,
-                                          const QString &tabName)
+                                          const QString &tabName,
+                                          DesignerWidgetFlags widgetFlags)
 {
     WidgetInfo widgetInfo;
 
@@ -135,6 +141,7 @@ WidgetInfo AbstractView::createWidgetInfo(QWidget *widget,
     widgetInfo.placementHint = placementHint;
     widgetInfo.placementPriority = placementPriority;
     widgetInfo.tabName = tabName;
+    widgetInfo.widgetFlags = widgetFlags;
 
     return widgetInfo;
 }
@@ -191,12 +198,63 @@ void AbstractView::modelAboutToBeDetached(Model *)
             Empty properties were removed.
 */
 
+void AbstractView::instancePropertyChanged(const QList<QPair<ModelNode, PropertyName> > &/*propertyList*/)
+{
+}
+
+void AbstractView::instanceInformationsChanged(const QMultiHash<ModelNode, InformationName> &/*informationChangeHash*/)
+{
+}
+
+void AbstractView::instancesRenderImageChanged(const QVector<ModelNode> &/*nodeList*/)
+{
+}
+
+void AbstractView::instancesPreviewImageChanged(const QVector<ModelNode> &/*nodeList*/)
+{
+}
+
+void AbstractView::instancesChildrenChanged(const QVector<ModelNode> &/*nodeList*/)
+{
+}
+
+void AbstractView::instancesToken(const QString &/*tokenName*/, int /*tokenNumber*/, const QVector<ModelNode> &/*nodeVector*/)
+{
+}
+
+void AbstractView::nodeSourceChanged(const ModelNode &/*modelNode*/, const QString &/*newNodeSource*/)
+{
+}
+
+void AbstractView::rewriterBeginTransaction()
+{
+}
+
+void AbstractView::rewriterEndTransaction()
+{
+}
+
+void AbstractView::instanceErrorChanged(const QVector<ModelNode> &/*errorNodeList*/)
+{
+}
+
+void AbstractView::instancesCompleted(const QVector<ModelNode> &/*completedNodeList*/)
+{
+}
+
 // Node related functions
 
 /*!
 \fn void AbstractView::nodeCreated(const ModelNode &createdNode)
 Called when the new node \a createdNode is created.
 */
+void AbstractView::nodeCreated(const ModelNode &/*createdNode*/)
+{
+}
+
+void AbstractView::currentStateChanged(const ModelNode &/*node*/)
+{
+}
 
 /*!
 Called when the file URL (that is needed to resolve relative paths against,
@@ -206,10 +264,25 @@ void AbstractView::fileUrlChanged(const QUrl &/*oldUrl*/, const QUrl &/*newUrl*/
 {
 }
 
+void AbstractView::nodeOrderChanged(const NodeListProperty &/*listProperty*/, const ModelNode &/*movedNode*/, int /*oldIndex*/)
+{
+}
+
 /*!
 \fn void AbstractView::nodeAboutToBeRemoved(const ModelNode &removedNode)
 Called when the node specified by \a removedNode will be removed.
 */
+void AbstractView::nodeAboutToBeRemoved(const ModelNode &/*removedNode*/)
+{
+}
+
+void AbstractView::nodeRemoved(const ModelNode &/*removedNode*/, const NodeAbstractProperty &/*parentProperty*/, PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& /*propertyList*/)
+{
+}
 
 /*!
 Called when the properties specified by \a propertyList are removed.
@@ -230,16 +303,68 @@ Called when the parent of \a node will be changed from \a oldPropertyParent to
 Called when the selection is changed from \a lastSelectedNodeList to
 \a selectedNodeList.
 */
+void AbstractView::selectedNodesChanged(const QList<ModelNode> &/*selectedNodeList*/, const QList<ModelNode> &/*lastSelectedNodeList*/)
+{
+}
 
-void AbstractView::auxiliaryDataChanged(const ModelNode &/*node*/, const PropertyName &/*name*/, const QVariant &/*data*/)
+void AbstractView::nodeAboutToBeReparented(const ModelNode &/*node*/, const NodeAbstractProperty &/*newPropertyParent*/, const NodeAbstractProperty &/*oldPropertyParent*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::nodeReparented(const ModelNode &/*node*/, const NodeAbstractProperty &/*newPropertyParent*/, const NodeAbstractProperty &/*oldPropertyParent*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::nodeIdChanged(const ModelNode& /*node*/, const QString& /*newId*/, const QString& /*oldId*/)
+{
+}
+
+void AbstractView::variantPropertiesChanged(const QList<VariantProperty>& /*propertyList*/, PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::bindingPropertiesChanged(const QList<BindingProperty>& /*propertyList*/, PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty>& /*propertyList*/, PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+void AbstractView::rootNodeTypeChanged(const QString &/*type*/, int /*majorVersion*/, int /*minorVersion*/)
+{
+}
+
+void AbstractView::nodeTypeChanged(const ModelNode & /*node*/, const TypeName & /*type*/, int /*majorVersion*/, int /*minorVersion*/)
 {
 
 }
 
+void AbstractView::importsChanged(const QList<Import> &/*addedImports*/, const QList<Import> &/*removedImports*/)
+{
+}
+
+void AbstractView::auxiliaryDataChanged(const ModelNode &/*node*/, const PropertyName &/*name*/, const QVariant &/*data*/)
+{
+}
+
 void AbstractView::customNotification(const AbstractView * /*view*/, const QString & /*identifier*/, const QList<ModelNode> & /*nodeList*/, const QList<QVariant> & /*data*/)
 {
 }
 
+void AbstractView::scriptFunctionsChanged(const ModelNode &/*node*/, const QStringList &/*scriptFunctionList*/)
+{
+}
+
+void AbstractView::documentMessagesChanged(const QList<DocumentMessage> &/*errors*/, const QList<DocumentMessage> &/*warnings*/)
+{
+}
+
+void AbstractView::currentTimelineChanged(const ModelNode & /*node*/)
+{
+
+}
+
 QList<ModelNode> AbstractView::toModelNodeList(const QList<Internal::InternalNode::Pointer> &nodeList) const
 {
     return QmlDesigner::toModelNodeList(nodeList, const_cast<AbstractView*>(this));
@@ -274,7 +399,7 @@ void AbstractView::setSelectedModelNodes(const QList<ModelNode> &selectedNodeLis
 
 void AbstractView::setSelectedModelNode(const ModelNode &modelNode)
 {
-    setSelectedModelNodes(QList<ModelNode>() << modelNode);
+    setSelectedModelNodes({modelNode});
 }
 
 /*!
@@ -312,7 +437,7 @@ QList<ModelNode> AbstractView::selectedModelNodes() const
 ModelNode AbstractView::firstSelectedModelNode() const
 {
     if (hasSelectedModelNodes())
-        return ModelNode(model()->d->selectedNodes().first(), model(), this);
+        return ModelNode(model()->d->selectedNodes().constFirst(), model(), this);
 
     return ModelNode();
 }
@@ -320,7 +445,7 @@ ModelNode AbstractView::firstSelectedModelNode() const
 ModelNode AbstractView::singleSelectedModelNode() const
 {
     if (hasSingleSelectedModelNode())
-        return ModelNode(model()->d->selectedNodes().first(), model(), this);
+        return ModelNode(model()->d->selectedNodes().constFirst(), model(), this);
 
     return ModelNode();
 }
@@ -352,7 +477,7 @@ bool AbstractView::hasId(const QString &id) const
     return model()->d->hasId(id);
 }
 
-QString firstCharToLower(const QString string)
+QString firstCharToLower(const QString &string)
 {
     QString resultString = string;
 
@@ -362,17 +487,28 @@ QString firstCharToLower(const QString string)
     return resultString;
 }
 
-QString AbstractView::generateNewId(const QString prefixName) const
+QString AbstractView::generateNewId(const QString &prefixName) const
 {
+    QString fixedPrefix = firstCharToLower(prefixName);
+    fixedPrefix.remove(' ');
+    if (!ModelNode::isValidId(fixedPrefix))
+        return generateNewId("element");
     int counter = 1;
 
-    QString newId = QString("%1%2").arg(firstCharToLower(prefixName)).arg(counter);
-    newId.remove(QRegExp(QLatin1String("[^a-zA-Z0-9_]")));
+    /* First try just the prefixName without number as postfix, then continue with 2 and further as postfix
+     * until id does not already exist.
+     * Properties of the root node are not allowed for ids, because they are available in the complete context
+     * without qualification.
+     * The id "item" is explicitly not allowed, because it is too likely to clash.
+    */
 
-    while (hasId(newId)) {
+    QString newId = QString(QStringLiteral("%1")).arg(firstCharToLower(prefixName));
+    newId.remove(QRegExp(QStringLiteral("[^a-zA-Z0-9_]")));
+
+    while (!ModelNode::isValidId(newId) || hasId(newId) || rootModelNode().hasProperty(newId.toUtf8()) || newId == "item") {
         counter += 1;
-        newId = QString("%1%2").arg(firstCharToLower(prefixName)).arg(counter);
-        newId.remove(QRegExp(QLatin1String("[^a-zA-Z0-9_]")));
+        newId = QString(QStringLiteral("%1%2")).arg(firstCharToLower(prefixName)).arg(counter - 1);
+        newId.remove(QRegExp(QStringLiteral("[^a-zA-Z0-9_]")));
     }
 
     return newId;
@@ -393,7 +529,7 @@ NodeInstanceView *AbstractView::nodeInstanceView() const
     if (model())
         return model()->d->nodeInstanceView();
     else
-        return 0;
+        return nullptr;
 }
 
 RewriterView *AbstractView::rewriterView() const
@@ -401,7 +537,7 @@ RewriterView *AbstractView::rewriterView() const
     if (model())
         return model()->d->rewriterView();
     else
-        return 0;
+        return nullptr;
 }
 
 void AbstractView::resetView()
@@ -414,6 +550,11 @@ void AbstractView::resetView()
     currentModel->attachView(this);
 }
 
+void AbstractView::resetPuppet()
+{
+    emitCustomNotification(QStringLiteral("reset QmlPuppet"));
+}
+
 bool AbstractView::hasWidget() const
 {
     return false;
@@ -424,9 +565,72 @@ WidgetInfo AbstractView::widgetInfo()
     return createWidgetInfo();
 }
 
+void AbstractView::disableWidget()
+{
+    if (hasWidget() && widgetInfo().widgetFlags == DesignerWidgetFlags::DisableOnError)
+        widgetInfo().widget->setEnabled(false);
+}
+
+void AbstractView::enableWidget()
+{
+    if (hasWidget() && widgetInfo().widgetFlags == DesignerWidgetFlags::DisableOnError)
+        widgetInfo().widget->setEnabled(true);
+}
+
+void AbstractView::contextHelpId(const Core::IContext::HelpIdCallback &callback) const
+{
+#ifndef QMLDESIGNER_TEST
+    QmlDesignerPlugin::instance()->viewManager().qmlJSEditorHelpId(callback);
+#else
+    callback(QString());
+#endif
+}
+
+void AbstractView::activateTimeline(const ModelNode &timeline)
+{
+    if (currentTimeline().isValid())
+        currentTimeline().toogleRecording(false);
+
+    if (model())
+        model()->d->notifyCurrentTimelineChanged(timeline);
+}
+
+void AbstractView::activateTimelineRecording(const ModelNode &timeline)
+{
+    if (currentTimeline().isValid())
+        currentTimeline().toogleRecording(true);
+
+    Internal::WriteLocker locker(m_model.data());
+
+    if (model())
+        model()->d->notifyCurrentTimelineChanged(timeline);
+}
+
+void AbstractView::deactivateTimelineRecording()
+{
+    if (currentTimeline().isValid()) {
+        currentTimeline().toogleRecording(false);
+        currentTimeline().resetGroupRecording();
+    }
+
+    if (model())
+        model()->d->notifyCurrentTimelineChanged(ModelNode());
+}
+
 QList<ModelNode> AbstractView::allModelNodes() const
 {
-   return toModelNodeList(model()->d->allNodes());
+    return toModelNodeList(model()->d->allNodes());
+}
+
+void AbstractView::emitDocumentMessage(const QString &error)
+{
+    emitDocumentMessage({DocumentMessage(error)});
+}
+
+void AbstractView::emitDocumentMessage(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings)
+{
+    if (model())
+        model()->d->setDocumentMessages(errors, warnings);
 }
 
 void AbstractView::emitCustomNotification(const QString &identifier)
@@ -450,6 +654,12 @@ void AbstractView::emitInstancePropertyChange(const QList<QPair<ModelNode, Prope
         model()->d->notifyInstancePropertyChange(propertyList);
 }
 
+void AbstractView::emitInstanceErrorChange(const QVector<qint32> &instanceIds)
+{
+    if (model() && nodeInstanceView() == this)
+        model()->d->notifyInstanceErrorChange(instanceIds);
+}
+
 void AbstractView::emitInstancesCompleted(const QVector<ModelNode> &nodeVector)
 {
     if (model() && nodeInstanceView() == this)
@@ -531,13 +741,38 @@ QmlModelState AbstractView::currentState() const
     return QmlModelState(currentStateNode());
 }
 
+QmlTimeline AbstractView::currentTimeline() const
+{
+    if (model())
+        return QmlTimeline(ModelNode(m_model.data()->d->currentTimelineNode(),
+                                            m_model.data(),
+                                            const_cast<AbstractView*>(this)));
+
+    return QmlTimeline();
+}
+
+static int getMinorVersionFromImport(const Model *model)
+{
+    foreach (const Import &import, model->imports()) {
+        if (import.isLibraryImport() && import.url() == "QtQuick") {
+            const QString versionString = import.version();
+            if (versionString.contains(".")) {
+                const QString minorVersionString = versionString.split(".").constLast();
+                return minorVersionString.toInt();
+            }
+        }
+    }
+
+    return -1;
+}
+
 static int getMajorVersionFromImport(const Model *model)
 {
     foreach (const Import &import, model->imports()) {
-        if (import.isLibraryImport() && import.url() == QLatin1String("QtQuick")) {
+        if (import.isLibraryImport() && import.url() == QStringLiteral("QtQuick")) {
             const QString versionString = import.version();
-            if (versionString.contains(QLatin1String("."))) {
-                const QString majorVersionString = versionString.split(QLatin1String(".")).first();
+            if (versionString.contains(QStringLiteral("."))) {
+                const QString majorVersionString = versionString.split(QStringLiteral(".")).constFirst();
                 return majorVersionString.toInt();
             }
         }
@@ -549,12 +784,21 @@ static int getMajorVersionFromImport(const Model *model)
 static int getMajorVersionFromNode(const ModelNode &modelNode)
 {
     if (modelNode.metaInfo().isValid()) {
-        if (modelNode.type() == "QtQuick.QtObject" || modelNode.type() == "QtQuick.Item")
-            return modelNode.majorVersion();
+        foreach (const NodeMetaInfo &info,  modelNode.metaInfo().classHierarchy()) {
+            if (info.typeName() == "QtQuick.QtObject" || info.typeName() == "QtQuick.Item")
+                return info.majorVersion();
+        }
+    }
 
-        foreach (const NodeMetaInfo &superClass,  modelNode.metaInfo().superClasses()) {
-            if (modelNode.type() == "QtQuick.QtObject" || modelNode.type() == "QtQuick.Item")
-                return superClass.majorVersion();
+    return 1; //default
+}
+
+static int getMinorVersionFromNode(const ModelNode &modelNode)
+{
+    if (modelNode.metaInfo().isValid()) {
+        foreach (const NodeMetaInfo &info,  modelNode.metaInfo().classHierarchy()) {
+            if (info.typeName() == "QtQuick.QtObject" || info.typeName() == "QtQuick.Item")
+                return info.minorVersion();
         }
     }
 
@@ -570,4 +814,14 @@ int AbstractView::majorQtQuickVersion() const
     return getMajorVersionFromNode(rootModelNode());
 }
 
+int AbstractView::minorQtQuickVersion() const
+{
+    int minorVersionFromImport = getMinorVersionFromImport(model());
+    if (minorVersionFromImport >= 0)
+        return minorVersionFromImport;
+
+    return getMinorVersionFromNode(rootModelNode());
+}
+
+
 } // namespace QmlDesigner
