@@ -24,8 +24,10 @@
 ****************************************************************************/
 
 #include "designmodewidget.h"
+#include "switchsplittabwidget.h"
+
+#include <designeractionmanager.h>
 
-#include <coreplugin/outputpane.h>
 #include "qmldesignerplugin.h"
 #include "crumblebar.h"
 #include "documentwarningwidget.h"
@@ -33,8 +35,9 @@
 #include <texteditor/textdocument.h>
 #include <nodeinstanceview.h>
 #include <itemlibrarywidget.h>
-#include <theming.h>
+#include <theme.h>
 
+#include <coreplugin/outputpane.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/designmode.h>
@@ -45,7 +48,6 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/idocument.h>
 #include <coreplugin/inavigationwidgetfactory.h>
-#include <extensionsystem/pluginmanager.h>
 
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
@@ -69,6 +71,12 @@ const char SB_PROJECTS[] = "Projects";
 const char SB_FILESYSTEM[] = "FileSystem";
 const char SB_OPENDOCUMENTS[] = "OpenDocuments";
 
+static void hideToolButtons(QList<QToolButton*> &buttons)
+{
+    foreach (QToolButton *button, buttons)
+        button->hide();
+}
+
 namespace QmlDesigner {
 namespace Internal {
 
@@ -76,17 +84,14 @@ class ItemLibrarySideBarItem : public Core::SideBarItem
 {
 public:
     explicit ItemLibrarySideBarItem(QWidget *widget, const QString &id);
-    virtual ~ItemLibrarySideBarItem();
+    ~ItemLibrarySideBarItem() override;
 
-    virtual QList<QToolButton *> createToolBarWidgets();
+    QList<QToolButton *> createToolBarWidgets() override;
 };
 
 ItemLibrarySideBarItem::ItemLibrarySideBarItem(QWidget *widget, const QString &id) : Core::SideBarItem(widget, id) {}
 
-ItemLibrarySideBarItem::~ItemLibrarySideBarItem()
-{
-
-}
+ItemLibrarySideBarItem::~ItemLibrarySideBarItem() = default;
 
 QList<QToolButton *> ItemLibrarySideBarItem::createToolBarWidgets()
 {
@@ -97,9 +102,9 @@ class DesignerSideBarItem : public Core::SideBarItem
 {
 public:
     explicit DesignerSideBarItem(QWidget *widget, WidgetInfo::ToolBarWidgetFactoryInterface *createToolBarWidgets, const QString &id);
-    virtual ~DesignerSideBarItem();
+    ~DesignerSideBarItem() override;
 
-    virtual QList<QToolButton *> createToolBarWidgets();
+    QList<QToolButton *> createToolBarWidgets() override;
 
 private:
     WidgetInfo::ToolBarWidgetFactoryInterface *m_toolBarWidgetFactory;
@@ -125,17 +130,10 @@ QList<QToolButton *> DesignerSideBarItem::createToolBarWidgets()
 }
 
 // ---------- DesignModeWidget
-DesignModeWidget::DesignModeWidget(QWidget *parent)
-    : QWidget(parent)
-    , m_toolBar(new Core::EditorToolBar(this))
+DesignModeWidget::DesignModeWidget()
+    : m_toolBar(new Core::EditorToolBar(this))
     , m_crumbleBar(new CrumbleBar(this))
 {
-    connect(viewManager().nodeInstanceView(), &NodeInstanceView::qmlPuppetCrashed,
-        [=]() {
-            RewriterError error(tr("Qt Quick emulation layer crashed"));
-            updateErrorStatus(QList<RewriterError>() << error);
-        }
-    );
 }
 
 DesignModeWidget::~DesignModeWidget()
@@ -172,19 +170,30 @@ void DesignModeWidget::toggleRightSidebar()
         m_rightSideBar->setVisible(!m_rightSideBar->isVisible());
 }
 
-void DesignModeWidget::toggleSidebars()
+QWidget *DesignModeWidget::createProjectExplorerWidget(QWidget *parent)
 {
-    if (m_initStatus == Initializing)
-        return;
+    const QList<Core::INavigationWidgetFactory *> factories =
+            Core::INavigationWidgetFactory::allNavigationFactories();
 
-    m_showSidebars = !m_showSidebars;
+    Core::NavigationView navigationView;
+    navigationView.widget = nullptr;
 
-    if (m_leftSideBar)
-        m_leftSideBar->setVisible(m_showSidebars);
-    if (m_rightSideBar)
-        m_rightSideBar->setVisible(m_showSidebars);
-    if (m_topSideBar)
-        m_topSideBar->setVisible(m_showSidebars);
+    foreach (Core::INavigationWidgetFactory *factory, factories) {
+        if (factory->id() == "Projects") {
+            navigationView = factory->createWidget();
+            hideToolButtons(navigationView.dockToolBarWidgets);
+        }
+    }
+
+    if (navigationView.widget) {
+        QByteArray sheet = Utils::FileReader::fetchQrc(":/qmldesigner/stylesheet.css");
+        sheet += Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css");
+        sheet += "QLabel { background-color: #4f4f4f; }";
+        navigationView.widget->setStyleSheet(Theme::replaceCssColors(QString::fromUtf8(sheet)));
+        navigationView.widget->setParent(parent);
+    }
+
+    return navigationView.widget;
 }
 
 void DesignModeWidget::readSettings()
@@ -217,10 +226,8 @@ void DesignModeWidget::enableWidgets()
 {
     if (debug)
         qDebug() << Q_FUNC_INFO;
-    m_warningWidget->setVisible(false);
+
     viewManager().enableWidgets();
-    m_leftSideBar->setEnabled(true);
-    m_rightSideBar->setEnabled(true);
     m_isDisabled = false;
 }
 
@@ -230,34 +237,26 @@ void DesignModeWidget::disableWidgets()
         qDebug() << Q_FUNC_INFO;
 
     viewManager().disableWidgets();
-    m_leftSideBar->setEnabled(false);
-    m_rightSideBar->setEnabled(false);
     m_isDisabled = true;
 }
 
-void DesignModeWidget::updateErrorStatus(const QList<RewriterError> &errors)
+void DesignModeWidget::switchTextOrForm()
 {
-    if (debug)
-        qDebug() << Q_FUNC_INFO << errors.count();
-
-    if (m_isDisabled && errors.isEmpty()) {
-        enableWidgets();
-     } else if (!errors.isEmpty()) {
-        disableWidgets();
-        showErrorMessageBox(errors);
-    }
-}
-
-static void hideToolButtons(QList<QToolButton*> &buttons)
-{
-    foreach (QToolButton *button, buttons)
-        button->hide();
+    if (m_centralTabWidget->currentWidget() == viewManager().widget("TextEditor"))
+        m_centralTabWidget->switchTo(viewManager().widget("FormEditor"));
+    else
+        m_centralTabWidget->switchTo(viewManager().widget("TextEditor"));
 }
 
 void DesignModeWidget::setup()
 {
+    auto &actionManager = viewManager().designerActionManager();
+    actionManager.createDefaultDesignerActions();
+    actionManager.createDefaultAddResourceHandler();
+    actionManager.polishActions();
+
     QList<Core::INavigationWidgetFactory *> factories =
-            ExtensionSystem::PluginManager::getObjects<Core::INavigationWidgetFactory>();
+            Core::INavigationWidgetFactory::allNavigationFactories();
 
     QWidget *openDocumentsWidget = nullptr;
     QWidget *projectsExplorer = nullptr;
@@ -265,7 +264,7 @@ void DesignModeWidget::setup()
 
     foreach (Core::INavigationWidgetFactory *factory, factories) {
         Core::NavigationView navigationView;
-        navigationView.widget = 0;
+        navigationView.widget = nullptr;
         if (factory->id() == "Projects") {
             navigationView = factory->createWidget();
             projectsExplorer = navigationView.widget;
@@ -287,30 +286,24 @@ void DesignModeWidget::setup()
             QByteArray sheet = Utils::FileReader::fetchQrc(":/qmldesigner/stylesheet.css");
             sheet += Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css");
             sheet += "QLabel { background-color: #4f4f4f; }";
-            navigationView.widget->setStyleSheet(Theming::replaceCssColors(QString::fromUtf8(sheet)));
+            navigationView.widget->setStyleSheet(Theme::replaceCssColors(QString::fromUtf8(sheet)));
         }
     }
 
-    QToolBar *toolBar = new QToolBar;
+    auto toolBar = new QToolBar;
     toolBar->addAction(viewManager().componentViewAction());
     toolBar->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-    m_toolBar->addCenterToolBar(toolBar);
+    DesignerActionToolBar *designerToolBar = QmlDesignerPlugin::instance()->viewManager().designerActionManager().createToolBar(m_toolBar);
+
+    designerToolBar->layout()->addWidget(toolBar);
+
+    m_toolBar->addCenterToolBar(designerToolBar);
+
+    m_toolBar->setMinimumWidth(320);
 
     m_mainSplitter = new MiniSplitter(this);
     m_mainSplitter->setObjectName("mainSplitter");
 
-    // warning frame should be not in layout, but still child of the widget
-    m_warningWidget = new DocumentWarningWidget(this);
-    m_warningWidget->setVisible(false);
-    connect(m_warningWidget.data(), &DocumentWarningWidget::gotoCodeClicked, [=]
-        (const QString &filePath, int codeLine, int codeColumn) {
-        Q_UNUSED(filePath);
-
-        if (currentDesignDocument() && currentDesignDocument()->textEditor())
-            currentDesignDocument()->textEditor()->gotoLine(codeLine, codeColumn);
-        Core::ModeManager::activateMode(Core::Constants::MODE_EDIT);
-    });
-
     QList<Core::SideBarItem*> sideBarItems;
     QList<Core::SideBarItem*> leftSideBarItems;
     QList<Core::SideBarItem*> rightSideBarItems;
@@ -380,14 +373,13 @@ void DesignModeWidget::setup()
 
     // Finishing touches:
     m_mainSplitter->setStretchFactor(1, 1);
-    m_mainSplitter->setSizes(QList<int>() << 150 << 300 << 150);
+    m_mainSplitter->setSizes({150, 300, 150});
 
     QLayout *mainLayout = new QBoxLayout(QBoxLayout::RightToLeft, this);
     mainLayout->setMargin(0);
     mainLayout->setSpacing(0);
     mainLayout->addWidget(m_mainSplitter);
 
-    m_warningWidget->setVisible(false);
     viewManager().enableWidgets();
     m_leftSideBar->setEnabled(true);
     m_rightSideBar->setEnabled(true);
@@ -455,7 +447,7 @@ void DesignModeWidget::addNavigatorHistoryEntry(const Utils::FileName &fileName)
 
 static QTabWidget *createWidgetsInTabWidget(const QList<WidgetInfo> &widgetInfos)
 {
-    QTabWidget *tabWidget = new QTabWidget;
+    auto tabWidget = new QTabWidget;
 
     foreach (const WidgetInfo &widgetInfo, widgetInfos)
         tabWidget->addTab(widgetInfo.widget, widgetInfo.tabName);
@@ -463,55 +455,57 @@ static QTabWidget *createWidgetsInTabWidget(const QList<WidgetInfo> &widgetInfos
     return tabWidget;
 }
 
-static QWidget *createTopSideBarWidget(const QList<WidgetInfo> &widgetInfos)
+static QWidget *createbottomSideBarWidget(const QList<WidgetInfo> &widgetInfos)
 {
     //### we now own these here
     QList<WidgetInfo> topWidgetInfos;
     foreach (const WidgetInfo &widgetInfo, widgetInfos) {
-        if (widgetInfo.placementHint == widgetInfo.TopPane)
+        if (widgetInfo.placementHint == widgetInfo.BottomPane)
             topWidgetInfos.append(widgetInfo);
     }
 
-    if (topWidgetInfos.count() == 1)
-        return topWidgetInfos.first().widget;
-    else
-        return createWidgetsInTabWidget(topWidgetInfos);
+    QWidget *widget = topWidgetInfos.constFirst().widget;
+    if (topWidgetInfos.count() > 1) {
+        QWidget *background = new QWidget();
+        background->setProperty("designerBackgroundColor", true);
+
+        QString sheet = QString::fromUtf8(Utils::FileReader::fetchQrc(":/qmldesigner/stylesheet.css"));
+        sheet.prepend("QWidget[designerBackgroundColor=\"true\"] {background-color: creatorTheme.QmlDesigner_BackgroundColorDarkAlternate;}");
+
+        background->setStyleSheet(Theme::replaceCssColors(sheet));
+        background->setLayout(new QVBoxLayout);
+        background->layout()->setContentsMargins(0, 0, 0, 0);
+
+        background->layout()->addWidget(createWidgetsInTabWidget(topWidgetInfos));
+        widget = background;
+    }
+    return widget;
 }
 
 static Core::MiniSplitter *createCentralSplitter(const QList<WidgetInfo> &widgetInfos)
 {
-    QList<WidgetInfo> centralWidgetInfos;
-    foreach (const WidgetInfo &widgetInfo, widgetInfos) {
-        if (widgetInfo.placementHint == widgetInfo.CentralPane)
-            centralWidgetInfos.append(widgetInfo);
-    }
-
     // editor and output panes
-    Core::MiniSplitter *outputPlaceholderSplitter = new Core::MiniSplitter;
-    outputPlaceholderSplitter->setStretchFactor(0, 10);
-    outputPlaceholderSplitter->setStretchFactor(1, 0);
+    auto outputPlaceholderSplitter = new Core::MiniSplitter;
     outputPlaceholderSplitter->setOrientation(Qt::Vertical);
 
+    auto switchSplitTabWidget = new SwitchSplitTabWidget();
+
+    foreach (const WidgetInfo &widgetInfo, widgetInfos) {
+        if (widgetInfo.placementHint == widgetInfo.CentralPane)
+            switchSplitTabWidget->addTab(widgetInfo.widget, widgetInfo.tabName);
+    }
+
+    outputPlaceholderSplitter->addWidget(switchSplitTabWidget);
+
+    QWidget *bottomSideBar = createbottomSideBarWidget(widgetInfos);
+    bottomSideBar->setObjectName("bottomSideBar");
+    outputPlaceholderSplitter->addWidget(bottomSideBar);
+
     auto outputPanePlaceholder = new Core::OutputPanePlaceHolder(Core::Constants::MODE_DESIGN, outputPlaceholderSplitter);
-
-    QTabWidget* tabWidget = createWidgetsInTabWidget(centralWidgetInfos);
-    tabWidget->setObjectName("centralTabWidget");
-    tabWidget->setTabPosition(QTabWidget::East);
-    tabWidget->tabBar()->setObjectName("centralTabBar");
-    tabWidget->setTabBarAutoHide(true);
-
-    QWidget *backgroundWidget = new QWidget();
-    backgroundWidget->setObjectName("backgroundWidget");
-    backgroundWidget->setLayout(new QVBoxLayout());
-    backgroundWidget->layout()->setMargin(0);
-    backgroundWidget->layout()->addWidget(tabWidget);
-
-    QByteArray sheet = Utils::FileReader::fetchQrc(":/qmldesigner/centerwidget.css");
-    backgroundWidget->setStyleSheet(Theming::replaceCssColors(QString::fromUtf8(sheet)));
-
-    outputPlaceholderSplitter->addWidget(backgroundWidget);
     outputPlaceholderSplitter->addWidget(outputPanePlaceholder);
 
+    outputPlaceholderSplitter->setStretchFactor(0, 10);
+    outputPlaceholderSplitter->setStretchFactor(1, 1);
     return outputPlaceholderSplitter;
 }
 
@@ -519,68 +513,53 @@ QWidget *DesignModeWidget::createCenterWidget()
 {
     QWidget *centerWidget = new QWidget;
 
-    QVBoxLayout *horizontalLayout = new QVBoxLayout(centerWidget);
+    auto horizontalLayout = new QVBoxLayout(centerWidget);
     horizontalLayout->setMargin(0);
     horizontalLayout->setSpacing(0);
 
     horizontalLayout->addWidget(m_toolBar);
     horizontalLayout->addWidget(createCrumbleBarFrame());
 
-    m_topSideBar = createTopSideBarWidget(viewManager().widgetInfos());
-    horizontalLayout->addWidget(m_topSideBar.data());
+    Core::MiniSplitter *centralSplitter = createCentralSplitter(viewManager().widgetInfos());
+    m_centralTabWidget = centralSplitter->findChild<SwitchSplitTabWidget*>();
+    Q_ASSERT(m_centralTabWidget);
+    m_centralTabWidget->switchTo(viewManager().widget("FormEditor"));
 
-    horizontalLayout->addWidget(createCentralSplitter(viewManager().widgetInfos()));
+    m_bottomSideBar = centralSplitter->findChild<QWidget*>("bottomSideBar");
+    Q_ASSERT(m_bottomSideBar);
+    horizontalLayout->addWidget(centralSplitter);
 
     return centerWidget;
 }
 
 QWidget *DesignModeWidget::createCrumbleBarFrame()
 {
-    QFrame *frame = new QFrame(this);
-    frame->setStyleSheet("background-color: #4e4e4e;");
-    frame->setFrameShape(QFrame::NoFrame);
-    QHBoxLayout *layout = new QHBoxLayout(frame);
+    auto frame = new Utils::StyledBar(this);
+    frame->setSingleRow(false);
+    auto layout = new QHBoxLayout(frame);
     layout->setMargin(0);
     layout->setSpacing(0);
-    frame->setLayout(layout);
     layout->addWidget(m_crumbleBar->crumblePath());
-    frame->setProperty("panelwidget", true);
-    frame->setProperty("panelwidget_singlerow", false);
 
     return frame;
 }
 
-void DesignModeWidget::showErrorMessageBox(const QList<RewriterError> &errors)
-{
-    Q_ASSERT(!errors.isEmpty());
-    m_warningWidget->setErrors(errors);
-    m_warningWidget->setVisible(true);
-}
-
-void DesignModeWidget::showWarningMessageBox(const QList<RewriterError> &warnings)
-{
-    Q_ASSERT(!warnings.isEmpty());
-    m_warningWidget->setWarnings(warnings);
-    m_warningWidget->setVisible(true);
-}
-
-bool DesignModeWidget::gotoCodeWasClicked()
-{
-    if (m_warningWidget)
-        return m_warningWidget->gotoCodeWasClicked();
-    return false;
-}
-
 CrumbleBar *DesignModeWidget::crumbleBar() const
 {
     return m_crumbleBar;
 }
 
-QString DesignModeWidget::contextHelpId() const
+void DesignModeWidget::showInternalTextEditor()
+{
+    m_centralTabWidget->switchTo(viewManager().widget("TextEditor"));
+}
+
+void DesignModeWidget::contextHelpId(const Core::IContext::HelpIdCallback &callback) const
 {
     if (currentDesignDocument())
-        return currentDesignDocument()->contextHelpId();
-    return QString();
+        currentDesignDocument()->contextHelpId(callback);
+    else
+        callback(QString());
 }
 
 void DesignModeWidget::initialize()
