@@ -23,45 +23,32 @@
 **
 ****************************************************************************/
 
-#ifndef QMLPROJECT_H
-#define QMLPROJECT_H
+#pragma once
 
 #include "qmlprojectmanager_global.h"
-
-#include "qmlprojectmanager.h"
 #include "qmlprojectnodes.h"
 
 #include <projectexplorer/project.h>
 
+#include <utils/environment.h>
+
 #include <QPointer>
 
 namespace ProjectExplorer { class RunConfiguration; }
-namespace QmlJS { class ModelManagerInterface; }
-namespace Utils { class FileSystemWatcher; }
 
 namespace QmlProjectManager {
 
 class QmlProjectItem;
 
-namespace Internal { class QmlProjectFile; }
-
 class QMLPROJECTMANAGER_EXPORT QmlProject : public ProjectExplorer::Project
 {
     Q_OBJECT
 
 public:
-    QmlProject(Internal::Manager *manager, const Utils::FileName &filename);
+    explicit QmlProject(const Utils::FileName &filename);
     ~QmlProject() override;
 
-    Utils::FileName filesFileName() const;
-
-    QString displayName() const override;
-    Internal::Manager *projectManager() const override;
-
-    bool supportsKit(ProjectExplorer::Kit *k, QString *errorMessage) const override;
-
-    Internal::QmlProjectNode *rootProjectNode() const override;
-    QStringList files(FilesMode fileMode) const override;
+    QList<ProjectExplorer::Task> projectIssues(const ProjectExplorer::Kit *k) const final;
 
     bool validProjectFile() const;
 
@@ -75,46 +62,43 @@ public:
 
     void refresh(RefreshOptions options);
 
-    QDir projectDir() const;
-    QStringList files() const;
+    Utils::FileName canonicalProjectDir() const;
     QString mainFile() const;
+    Utils::FileName targetDirectory(const ProjectExplorer::Target *target) const;
+    Utils::FileName targetFile(const Utils::FileName &sourceFile,
+                               const ProjectExplorer::Target *target) const;
+
+    QList<Utils::EnvironmentItem> environment() const;
     QStringList customImportPaths() const;
 
     bool addFiles(const QStringList &filePaths);
 
     void refreshProjectFile();
 
-    enum QmlImport { UnknownImport, QtQuick1Import, QtQuick2Import };
-    QmlImport defaultImport() const;
-
-private slots:
-    void refreshFiles(const QSet<QString> &added, const QSet<QString> &removed);
-    void addedTarget(ProjectExplorer::Target *target);
-    void onActiveTargetChanged(ProjectExplorer::Target *target);
-    void onKitChanged();
-    void addedRunConfiguration(ProjectExplorer::RunConfiguration *);
+    bool needsBuildConfigurations() const final;
 
+    static QStringList makeAbsolute(const Utils::FileName &path, const QStringList &relativePaths);
 protected:
     RestoreResult fromMap(const QVariantMap &map, QString *errorMessage) override;
 
 private:
+    void generateProjectTree();
+    void updateDeploymentData(ProjectExplorer::Target *target);
+    void refreshFiles(const QSet<QString> &added, const QSet<QString> &removed);
+    void refreshTargetDirectory();
+    void addedTarget(ProjectExplorer::Target *target);
+    void onActiveTargetChanged(ProjectExplorer::Target *target);
+    void onKitChanged();
+
     // plain format
     void parseProject(RefreshOptions options);
-    QStringList convertToAbsoluteFiles(const QStringList &paths) const;
-    QmlJS::ModelManagerInterface *modelManager() const;
 
-    QString m_projectName;
-    QmlImport m_defaultImport;
-    ProjectExplorer::Target *m_activeTarget = 0;
-
-    // plain format
-    QStringList m_files;
+    ProjectExplorer::Target *m_activeTarget = nullptr;
 
     QPointer<QmlProjectItem> m_projectItem;
+    Utils::FileName m_canonicalProjectDir;
 };
 
 } // namespace QmlProjectManager
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QmlProjectManager::QmlProject::RefreshOptions)
-
-#endif // QMLPROJECT_H
