@@ -50,12 +50,6 @@ AbstractRemoteLinuxDeployStep::AbstractRemoteLinuxDeployStep(BuildStepList *bsl,
 {
 }
 
-AbstractRemoteLinuxDeployStep::AbstractRemoteLinuxDeployStep(BuildStepList *bsl,
-        AbstractRemoteLinuxDeployStep *other)
-    : BuildStep(bsl, other), d(new Internal::AbstractRemoteLinuxDeployStepPrivate)
-{
-}
-
 AbstractRemoteLinuxDeployStep::~AbstractRemoteLinuxDeployStep()
 {
     delete d;
@@ -81,19 +75,24 @@ bool AbstractRemoteLinuxDeployStep::init(QList<const BuildStep *> &earlierSteps)
     deployService()->setTarget(target());
     const bool canDeploy = initInternal(&error);
     if (!canDeploy)
-        emit addOutput(tr("Cannot deploy: %1").arg(error), ErrorMessageOutput);
+        emit addOutput(tr("Cannot deploy: %1").arg(error), OutputFormat::ErrorMessage);
     return canDeploy;
 }
 
 void AbstractRemoteLinuxDeployStep::run(QFutureInterface<bool> &fi)
 {
-    connect(deployService(), SIGNAL(errorMessage(QString)), SLOT(handleErrorMessage(QString)));
-    connect(deployService(), SIGNAL(progressMessage(QString)),
-        SLOT(handleProgressMessage(QString)));
-    connect(deployService(), SIGNAL(warningMessage(QString)), SLOT(handleWarningMessage(QString)));
-    connect(deployService(), SIGNAL(stdOutData(QString)), SLOT(handleStdOutData(QString)));
-    connect(deployService(), SIGNAL(stdErrData(QString)), SLOT(handleStdErrData(QString)));
-    connect(deployService(), SIGNAL(finished()), SLOT(handleFinished()));
+    connect(deployService(), &AbstractRemoteLinuxDeployService::errorMessage,
+            this, &AbstractRemoteLinuxDeployStep::handleErrorMessage);
+    connect(deployService(), &AbstractRemoteLinuxDeployService::progressMessage,
+            this, &AbstractRemoteLinuxDeployStep::handleProgressMessage);
+    connect(deployService(), &AbstractRemoteLinuxDeployService::warningMessage,
+            this, &AbstractRemoteLinuxDeployStep::handleWarningMessage);
+    connect(deployService(), &AbstractRemoteLinuxDeployService::stdOutData,
+            this, &AbstractRemoteLinuxDeployStep::handleStdOutData);
+    connect(deployService(), &AbstractRemoteLinuxDeployService::stdErrData,
+            this, &AbstractRemoteLinuxDeployStep::handleStdErrData);
+    connect(deployService(), &AbstractRemoteLinuxDeployService::finished,
+            this, &AbstractRemoteLinuxDeployStep::handleFinished);
 
     d->hasError = false;
     d->future = fi;
@@ -105,7 +104,8 @@ void AbstractRemoteLinuxDeployStep::cancel()
     if (d->hasError)
         return;
 
-    emit addOutput(tr("User requests deployment to stop; cleaning up."), MessageOutput);
+    emit addOutput(tr("User requests deployment to stop; cleaning up."),
+                   OutputFormat::NormalMessage);
     d->hasError = true;
     deployService()->stop();
 }
@@ -122,7 +122,7 @@ RemoteLinuxDeployConfiguration *AbstractRemoteLinuxDeployStep::deployConfigurati
 
 void AbstractRemoteLinuxDeployStep::handleProgressMessage(const QString &message)
 {
-    emit addOutput(message, MessageOutput);
+    emit addOutput(message, OutputFormat::NormalMessage);
 }
 
 void AbstractRemoteLinuxDeployStep::handleErrorMessage(const QString &message)
@@ -130,7 +130,7 @@ void AbstractRemoteLinuxDeployStep::handleErrorMessage(const QString &message)
     ProjectExplorer::Task task = Task(Task::Error, message, Utils::FileName(), -1,
                                       Constants::TASK_CATEGORY_DEPLOYMENT);
     emit addTask(task, 1); // TODO correct?
-    emit addOutput(message, ErrorMessageOutput);
+    emit addOutput(message, OutputFormat::ErrorMessage);
     d->hasError = true;
 }
 
@@ -139,28 +139,27 @@ void AbstractRemoteLinuxDeployStep::handleWarningMessage(const QString &message)
     ProjectExplorer::Task task = Task(Task::Warning, message, Utils::FileName(), -1,
                                       Constants::TASK_CATEGORY_DEPLOYMENT);
     emit addTask(task, 1); // TODO correct?
-    emit addOutput(message, ErrorMessageOutput);
+    emit addOutput(message, OutputFormat::ErrorMessage);
 }
 
 void AbstractRemoteLinuxDeployStep::handleFinished()
 {
     if (d->hasError)
-        emit addOutput(tr("Deploy step failed."), ErrorMessageOutput);
+        emit addOutput(tr("Deploy step failed."), OutputFormat::ErrorMessage);
     else
-        emit addOutput(tr("Deploy step finished."), MessageOutput);
+        emit addOutput(tr("Deploy step finished."), OutputFormat::NormalMessage);
     disconnect(deployService(), 0, this, 0);
-    d->future.reportResult(!d->hasError);
-    emit finished();
+    reportRunResult(d->future, !d->hasError);
 }
 
 void AbstractRemoteLinuxDeployStep::handleStdOutData(const QString &data)
 {
-    emit addOutput(data, NormalOutput, DontAppendNewline);
+    emit addOutput(data, OutputFormat::Stdout, DontAppendNewline);
 }
 
 void AbstractRemoteLinuxDeployStep::handleStdErrData(const QString &data)
 {
-    emit addOutput(data, ErrorOutput, DontAppendNewline);
+    emit addOutput(data, OutputFormat::Stderr, DontAppendNewline);
 }
 
 } // namespace RemoteLinux
