@@ -54,10 +54,8 @@ namespace Internal {
 class FormatsModel : public QAbstractListModel
 {
 public:
-    FormatsModel(QObject *parent = 0):
-        QAbstractListModel(parent),
-        m_descriptions(0),
-        m_scheme(0)
+    FormatsModel(QObject *parent = nullptr):
+        QAbstractListModel(parent)
     {
     }
 
@@ -82,12 +80,12 @@ public:
         emitDataChanged(index(0));
     }
 
-    int rowCount(const QModelIndex &parent) const
+    int rowCount(const QModelIndex &parent) const override
     {
         return (parent.isValid() || !m_descriptions) ? 0 : int(m_descriptions->size());
     }
 
-    QVariant data(const QModelIndex &index, int role) const
+    QVariant data(const QModelIndex &index, int role) const override
     {
         if (!m_descriptions || !m_scheme)
             return QVariant();
@@ -139,8 +137,8 @@ public:
     }
 
 private:
-    const FormatDescriptions *m_descriptions;
-    const ColorScheme *m_scheme;
+    const FormatDescriptions *m_descriptions = nullptr;
+    const ColorScheme *m_scheme = nullptr;
     QFont m_baseFont;
 };
 
@@ -154,6 +152,8 @@ ColorSchemeEdit::ColorSchemeEdit(QWidget *parent) :
 {
     setContentsMargins(0, layoutSpacing, 0, 0);
     m_ui->setupUi(this);
+    m_ui->detailsScrollArea->viewport()->setAutoFillBackground(false);
+    m_ui->scrollAreaWidgetContents->setAutoFillBackground(false);
     m_ui->itemList->setModel(m_formatsModel);
 
     populateUnderlineStyleComboBox();
@@ -280,6 +280,7 @@ void ColorSchemeEdit::updateForegroundControls()
 
     bool isVisible = formatDescription.showControl(FormatDescription::ShowForegroundControl);
 
+    m_ui->relativeForegroundHeadline->setEnabled(isVisible);
     m_ui->foregroundLabel->setVisible(isVisible);
     m_ui->foregroundToolButton->setVisible(isVisible);
     m_ui->eraseForegroundToolButton->setVisible(isVisible);
@@ -298,6 +299,7 @@ void ColorSchemeEdit::updateBackgroundControls()
 
     bool isVisible = formatDescription.showControl(FormatDescription::ShowBackgroundControl);
 
+    m_ui->relativeBackgroundHeadline->setVisible(isVisible);
     m_ui->backgroundLabel->setVisible(isVisible);
     m_ui->backgroundToolButton->setVisible(isVisible);
     m_ui->eraseBackgroundToolButton->setVisible(isVisible);
@@ -328,6 +330,14 @@ void ColorSchemeEdit::updateRelativeForegroundControls()
     m_ui->relativeForegroundSpacer2->setVisible(isVisible);
     m_ui->relativeForegroundSpacer3->setVisible(isVisible);
 
+    bool isEnabled = !m_readOnly && !format.foreground().isValid();
+
+    m_ui->relativeForegroundHeadline->setEnabled(isEnabled);
+    m_ui->foregroundSaturationLabel->setEnabled(isEnabled);
+    m_ui->foregroundLightnessLabel->setEnabled(isEnabled);
+    m_ui->foregroundSaturationSpinBox->setEnabled(isEnabled);
+    m_ui->foregroundLightnessSpinBox->setEnabled(isEnabled);
+
     m_ui->foregroundSaturationSpinBox->setValue(format.relativeForegroundSaturation());
     m_ui->foregroundLightnessSpinBox->setValue(format.relativeForegroundLightness());
 }
@@ -351,6 +361,14 @@ void ColorSchemeEdit::updateRelativeBackgroundControls()
     m_ui->relativeBackgroundSpacer2->setVisible(isVisible);
     m_ui->relativeBackgroundSpacer3->setVisible(isVisible);
 
+    bool isEnabled = !m_readOnly && !format.background().isValid();
+
+    m_ui->relativeBackgroundHeadline->setEnabled(isEnabled);
+    m_ui->backgroundSaturationLabel->setEnabled(isEnabled);
+    m_ui->backgroundLightnessLabel->setEnabled(isEnabled);
+    m_ui->backgroundSaturationSpinBox->setEnabled(isEnabled);
+    m_ui->backgroundLightnessSpinBox->setEnabled(isEnabled);
+
     m_ui->backgroundSaturationSpinBox->setValue(format.relativeBackgroundSaturation());
     m_ui->backgroundLightnessSpinBox->setValue(format.relativeBackgroundLightness());
 }
@@ -417,6 +435,8 @@ void ColorSchemeEdit::changeForeColor()
         m_scheme.formatFor(category).setForeground(newColor);
         m_formatsModel->emitDataChanged(index);
     }
+
+    updateControls();
 }
 
 void ColorSchemeEdit::changeBackColor()
@@ -438,6 +458,8 @@ void ColorSchemeEdit::changeBackColor()
         if (index.row() == 0)
             setItemListBackground(newColor);
     }
+
+    updateControls();
 }
 
 void ColorSchemeEdit::eraseBackColor()
@@ -453,6 +475,8 @@ void ColorSchemeEdit::eraseBackColor()
         m_scheme.formatFor(category).setBackground(newColor);
         m_formatsModel->emitDataChanged(index);
     }
+
+    updateControls();
 }
 
 void ColorSchemeEdit::eraseForeColor()
@@ -468,6 +492,8 @@ void ColorSchemeEdit::eraseForeColor()
         m_scheme.formatFor(category).setForeground(newColor);
         m_formatsModel->emitDataChanged(index);
     }
+
+    updateControls();
 }
 
 void ColorSchemeEdit::changeRelativeForeColor()
