@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -32,23 +27,23 @@
 #include "vcsbasesubmiteditor.h"
 #include "vcsplugin.h"
 #include "commonvcssettings.h"
-#include "vcsbaseoutputwindow.h"
-#include "corelistener.h"
-#include "command.h"
+#include "vcsoutputwindow.h"
+#include "vcscommand.h"
 
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/id.h>
-#include <coreplugin/iversioncontrol.h>
+#include <coreplugin/idocument.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <coreplugin/vcsmanager.h>
-#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/session.h>
 #include <utils/qtcassert.h>
 #include <utils/synchronousprocess.h>
 
 #include <QDebug>
 #include <QDir>
+#include <QLoggingCategory>
 #include <QSharedData>
 #include <QScopedPointer>
 #include <QSharedPointer>
@@ -59,9 +54,15 @@
 #include <QMessageBox>
 #include <QFileDialog>
 
+using namespace Core;
 using namespace Utils;
+using namespace ProjectExplorer;
 
-enum { debug = 0, debugRepositorySearch = 0 };
+namespace {
+Q_LOGGING_CATEGORY(baseLog, "qtc.vcs.base", QtWarningMsg)
+Q_LOGGING_CATEGORY(findRepoLog, "qtc.vcs.find-repo", QtWarningMsg)
+Q_LOGGING_CATEGORY(stateLog, "qtc.vcs.state", QtWarningMsg)
+}
 
 /*!
     \namespace VcsBase
@@ -87,8 +88,9 @@ namespace Internal {
     Aggregated in the QSharedData of VcsBase::VcsBasePluginState.
 */
 
-struct State
+class State
 {
+public:
     void clearFile();
     void clearPatchFile();
     void clearProject();
@@ -197,31 +199,55 @@ class StateListener : public QObject
 public:
     explicit StateListener(QObject *parent);
 
+    static QString windowTitleVcsTopic(const QString &filePath);
+
 signals:
-    void stateChanged(const VcsBase::Internal::State &s, Core::IVersionControl *vc);
+    void stateChanged(const VcsBase::Internal::State &s, IVersionControl *vc);
 
 public slots:
     void slotStateChanged();
 };
 
-StateListener::StateListener(QObject *parent) :
-        QObject(parent)
+StateListener::StateListener(QObject *parent) : QObject(parent)
 {
-    connect(Core::EditorManager::instance(), SIGNAL(currentEditorChanged(Core::IEditor*)),
-            this, SLOT(slotStateChanged()));
-    connect(Core::EditorManager::instance(), SIGNAL(currentDocumentStateChanged()),
-            this, SLOT(slotStateChanged()));
-    connect(Core::VcsManager::instance(), SIGNAL(repositoryChanged(QString)),
-            this, SLOT(slotStateChanged()));
+    connect(EditorManager::instance(), &EditorManager::currentEditorChanged,
+            this, &StateListener::slotStateChanged);
+    connect(EditorManager::instance(), &EditorManager::currentDocumentStateChanged,
+            this, &StateListener::slotStateChanged);
+    connect(VcsManager::instance(), &VcsManager::repositoryChanged,
+            this, &StateListener::slotStateChanged);
 
-    connect(ProjectExplorer::ProjectExplorerPlugin::instance(),
-            SIGNAL(currentProjectChanged(ProjectExplorer::Project*)),
-            this, SLOT(slotStateChanged()));
+    connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
+            this, &StateListener::slotStateChanged);
+    connect(SessionManager::instance(), &SessionManager::startupProjectChanged,
+            this, &StateListener::slotStateChanged);
+
+
+    EditorManager::setWindowTitleVcsTopicHandler(&StateListener::windowTitleVcsTopic);
+}
+
+QString StateListener::windowTitleVcsTopic(const QString &filePath)
+{
+    QString searchPath;
+    if (!filePath.isEmpty()) {
+        searchPath = QFileInfo(filePath).absolutePath();
+    } else {
+        // use single project's information if there is only one loaded.
+        const QList<Project *> projects = SessionManager::projects();
+        if (projects.size() == 1)
+            searchPath = projects.first()->projectDirectory().toString();
+    }
+    if (searchPath.isEmpty())
+        return QString();
+    QString topLevelPath;
+    IVersionControl *vc = VcsManager::findVersionControlForDirectory(
+                searchPath, &topLevelPath);
+    return (vc && !topLevelPath.isEmpty()) ? vc->vcsTopic(topLevelPath) : QString();
 }
 
 static inline QString displayNameOfEditor(const QString &fileName)
 {
-    Core::IDocument *document = Core::DocumentModel::documentForFilePath(fileName);
+    IDocument *document = DocumentModel::documentForFilePath(fileName);
     if (document)
         return document->displayName();
     return QString();
@@ -233,61 +259,57 @@ void StateListener::slotStateChanged()
     // temporary path prefix or does the file contains a hash, indicating a project
     // folder?
     State state;
-    Core::IDocument *currentDocument = Core::EditorManager::currentDocument();
-    if (!currentDocument) {
-        state.currentFile.clear();
-    } else {
-        state.currentFile = currentDocument->filePath();
+    IDocument *currentDocument = EditorManager::currentDocument();
+    if (currentDocument) {
+        state.currentFile = currentDocument->filePath().toString();
         if (state.currentFile.isEmpty() || currentDocument->isTemporary())
             state.currentFile = VcsBasePlugin::source(currentDocument);
     }
-    QScopedPointer<QFileInfo> currentFileInfo; // Instantiate QFileInfo only once if required.
-    if (!state.currentFile.isEmpty()) {
-        const bool isTempFile = state.currentFile.startsWith(QDir::tempPath());
-        // Quick check: Does it look like a patch?
-        const bool isPatch = state.currentFile.endsWith(QLatin1String(".patch"))
-                             || state.currentFile.endsWith(QLatin1String(".diff"));
-        if (isPatch) {
-            // Patch: Figure out a name to display. If it is a temp file, it could be
-            // Codepaster. Use the display name of the editor.
-            state.currentPatchFile = state.currentFile;
-            if (isTempFile)
-                state.currentPatchFileDisplayName = displayNameOfEditor(state.currentPatchFile);
-            if (state.currentPatchFileDisplayName.isEmpty()) {
-                currentFileInfo.reset(new QFileInfo(state.currentFile));
-                state.currentPatchFileDisplayName = currentFileInfo->fileName();
-            }
-        }
-        // For actual version control operations on it:
-        // Do not show temporary files and project folders ('#')
-        if (isTempFile || state.currentFile.contains(QLatin1Char('#')))
-            state.currentFile.clear();
-    }
 
     // Get the file and its control. Do not use the file unless we find one
-    Core::IVersionControl *fileControl = 0;
+    IVersionControl *fileControl = nullptr;
+
     if (!state.currentFile.isEmpty()) {
-        if (currentFileInfo.isNull())
-            currentFileInfo.reset(new QFileInfo(state.currentFile));
-        if (currentFileInfo->isDir()) {
-            state.currentFile.clear();
-            state.currentFileDirectory = currentFileInfo->absoluteFilePath();
-        } else {
-            state.currentFileDirectory = currentFileInfo->absolutePath();
-            state.currentFileName = currentFileInfo->fileName();
+        QFileInfo currentFi(state.currentFile);
+
+        if (currentFi.exists()) {
+            // Quick check: Does it look like a patch?
+            const bool isPatch = state.currentFile.endsWith(".patch")
+                    || state.currentFile.endsWith(".diff");
+            if (isPatch) {
+                // Patch: Figure out a name to display. If it is a temp file, it could be
+                // Codepaster. Use the display name of the editor.
+                state.currentPatchFile = state.currentFile;
+                state.currentPatchFileDisplayName = displayNameOfEditor(state.currentPatchFile);
+                if (state.currentPatchFileDisplayName.isEmpty())
+                    state.currentPatchFileDisplayName = currentFi.fileName();
+            }
+
+            if (currentFi.isDir()) {
+                state.currentFile.clear();
+                state.currentFileDirectory = currentFi.absoluteFilePath();
+            } else {
+                state.currentFileDirectory = currentFi.absolutePath();
+                state.currentFileName = currentFi.fileName();
+            }
+            fileControl = VcsManager::findVersionControlForDirectory(state.currentFileDirectory,
+                                                                     &state.currentFileTopLevel);
         }
-        fileControl = Core::VcsManager::findVersionControlForDirectory(
-                    state.currentFileDirectory,
-                    &state.currentFileTopLevel);
+
         if (!fileControl)
             state.clearFile();
     }
+
     // Check for project, find the control
-    Core::IVersionControl *projectControl = 0;
-    if (const ProjectExplorer::Project *currentProject = ProjectExplorer::ProjectExplorerPlugin::currentProject()) {
+    IVersionControl *projectControl = nullptr;
+    Project *currentProject = ProjectTree::currentProject();
+    if (!currentProject)
+        currentProject = SessionManager::startupProject();
+
+    if (currentProject) {
         state.currentProjectPath = currentProject->projectDirectory().toString();
         state.currentProjectName = currentProject->displayName();
-        projectControl = Core::VcsManager::findVersionControlForDirectory(state.currentProjectPath,
+        projectControl = VcsManager::findVersionControlForDirectory(state.currentProjectPath,
                                                                     &state.currentProjectTopLevel);
         if (projectControl) {
             // If we have both, let the file's one take preference
@@ -297,16 +319,16 @@ void StateListener::slotStateChanged()
             state.clearProject(); // No control found
         }
     }
+
     // Assemble state and emit signal.
-    Core::IVersionControl *vc = fileControl;
+    IVersionControl *vc = fileControl;
     if (!vc)
         vc = projectControl;
-    if (!vc) {
+    if (!vc)
         state.clearPatchFile(); // Need a repository to patch
-        Core::EditorManager::setWindowTitleVcsTopic(QString());
-    }
-    if (debug)
-        qDebug() << state << (vc ? vc->displayName() : QLatin1String("No version control"));
+
+    qCDebug(stateLog).noquote() << "VC:" << (vc ? vc->displayName() : QString("None")) << state;
+    EditorManager::updateWindowTitles();
     emit stateChanged(state, vc);
 }
 
@@ -335,12 +357,13 @@ public:
 */
 
 VcsBasePluginState::VcsBasePluginState() : data(new VcsBasePluginStateData)
-{
-}
+{ }
+
 
 VcsBasePluginState::VcsBasePluginState(const VcsBasePluginState &rhs) : data(rhs.data)
-{
-}
+{ }
+
+VcsBasePluginState::~VcsBasePluginState() = default;
 
 VcsBasePluginState &VcsBasePluginState::operator=(const VcsBasePluginState &rhs)
 {
@@ -349,10 +372,6 @@ VcsBasePluginState &VcsBasePluginState::operator=(const VcsBasePluginState &rhs)
     return *this;
 }
 
-VcsBasePluginState::~VcsBasePluginState()
-{
-}
-
 QString VcsBasePluginState::currentFile() const
 {
     return data->m_state.currentFile;
@@ -500,61 +519,53 @@ VCSBASE_EXPORT QDebug operator<<(QDebug in, const VcsBasePluginState &state)
 class VcsBasePluginPrivate
 {
 public:
-    explicit VcsBasePluginPrivate();
-
     inline bool supportsRepositoryCreation() const;
 
     QPointer<VcsBaseSubmitEditor> m_submitEditor;
-    Core::IVersionControl *m_versionControl;
+    IVersionControl *m_versionControl = nullptr;
+    Context m_context;
     VcsBasePluginState m_state;
-    int m_actionState;
+    int m_actionState = -1;
 
     static Internal::StateListener *m_listener;
 };
 
-VcsBasePluginPrivate::VcsBasePluginPrivate() :
-    m_versionControl(0),
-    m_actionState(-1)
-{
-}
-
 bool VcsBasePluginPrivate::supportsRepositoryCreation() const
 {
-    return m_versionControl && m_versionControl->supportsOperation(Core::IVersionControl::CreateRepositoryOperation);
+    return m_versionControl && m_versionControl->supportsOperation(IVersionControl::CreateRepositoryOperation);
 }
 
-Internal::StateListener *VcsBasePluginPrivate::m_listener = 0;
+Internal::StateListener *VcsBasePluginPrivate::m_listener = nullptr;
 
 VcsBasePlugin::VcsBasePlugin() :
     d(new VcsBasePluginPrivate())
-{
-}
+{ }
 
 VcsBasePlugin::~VcsBasePlugin()
 {
     delete d;
 }
 
-void VcsBasePlugin::initializeVcs(Core::IVersionControl *vc)
+void VcsBasePlugin::initializeVcs(IVersionControl *vc, const Context &context)
 {
+    QTC_ASSERT(vc, return);
+
     d->m_versionControl = vc;
-    addAutoReleasedObject(vc);
+    d->m_context = context;
 
     Internal::VcsPlugin *plugin = Internal::VcsPlugin::instance();
-    connect(plugin->coreListener(), SIGNAL(submitEditorAboutToClose(VcsBaseSubmitEditor*,bool*)),
-            this, SLOT(slotSubmitEditorAboutToClose(VcsBaseSubmitEditor*,bool*)));
+    connect(plugin, &Internal::VcsPlugin::submitEditorAboutToClose,
+            this, &VcsBasePlugin::slotSubmitEditorAboutToClose);
     // First time: create new listener
     if (!VcsBasePluginPrivate::m_listener)
         VcsBasePluginPrivate::m_listener = new Internal::StateListener(plugin);
-    connect(VcsBasePluginPrivate::m_listener,
-            SIGNAL(stateChanged(VcsBase::Internal::State,Core::IVersionControl*)),
-            this,
-            SLOT(slotStateChanged(VcsBase::Internal::State,Core::IVersionControl*)));
+    connect(VcsBasePluginPrivate::m_listener, &Internal::StateListener::stateChanged,
+            this, &VcsBasePlugin::slotStateChanged);
     // VCSes might have become (un-)available, so clear the VCS directory cache
-    connect(vc, SIGNAL(configurationChanged()),
-            Core::VcsManager::instance(), SLOT(clearVersionControlCache()));
-    connect(vc, SIGNAL(configurationChanged()),
-            VcsBasePluginPrivate::m_listener, SLOT(slotStateChanged()));
+    connect(vc, &IVersionControl::configurationChanged,
+            VcsManager::instance(), &VcsManager::clearVersionControlCache);
+    connect(vc, &IVersionControl::configurationChanged,
+            VcsBasePluginPrivate::m_listener, &Internal::StateListener::slotStateChanged);
 }
 
 void VcsBasePlugin::extensionsInitialized()
@@ -565,29 +576,28 @@ void VcsBasePlugin::extensionsInitialized()
 
 void VcsBasePlugin::slotSubmitEditorAboutToClose(VcsBaseSubmitEditor *submitEditor, bool *result)
 {
-    if (debug)
-        qDebug() << this << "plugin's submit editor"
-                 << d->m_submitEditor << (d->m_submitEditor ? d->m_submitEditor->document()->id().name() : "")
-                 << "closing submit editor" << submitEditor
-                 << (submitEditor ? submitEditor->document()->id().name() : "");
+    qCDebug(baseLog) << this << "plugin's submit editor" << d->m_submitEditor
+                     << (d->m_submitEditor ? d->m_submitEditor->document()->id().name() : QByteArray())
+                     << "closing submit editor" << submitEditor
+                     << (submitEditor ? submitEditor->document()->id().name() : QByteArray());
     if (submitEditor == d->m_submitEditor)
         *result = submitEditorAboutToClose();
 }
 
-Core::IVersionControl *VcsBasePlugin::versionControl() const
+IVersionControl *VcsBasePlugin::versionControl() const
 {
     return d->m_versionControl;
 }
 
-void VcsBasePlugin::slotStateChanged(const VcsBase::Internal::State &newInternalState, Core::IVersionControl *vc)
+void VcsBasePlugin::slotStateChanged(const VcsBase::Internal::State &newInternalState, IVersionControl *vc)
 {
     if (vc == d->m_versionControl) {
         // We are directly affected: Change state
         if (!d->m_state.equals(newInternalState)) {
             d->m_state.setState(newInternalState);
             updateActions(VcsEnabled);
+            ICore::addAdditionalContext(d->m_context);
         }
-        Core::EditorManager::setWindowTitleVcsTopic(vc->vcsTopic(d->m_state.topLevel()));
     } else {
         // Some other VCS plugin or state changed: Reset us to empty state.
         const ActionState newActionState = vc ? OtherVcsEnabled : NoVcsEnabled;
@@ -597,6 +607,7 @@ void VcsBasePlugin::slotStateChanged(const VcsBase::Internal::State &newInternal
             d->m_state = emptyState;
             updateActions(newActionState);
         }
+        ICore::removeAdditionalContext(d->m_context);
     }
 }
 
@@ -607,19 +618,18 @@ const VcsBasePluginState &VcsBasePlugin::currentState() const
 
 bool VcsBasePlugin::enableMenuAction(ActionState as, QAction *menuAction) const
 {
-    if (debug)
-        qDebug() << "enableMenuAction" << menuAction->text() << as;
+    qCDebug(baseLog) << "enableMenuAction" << menuAction->text() << as;
     switch (as) {
-    case VcsBase::VcsBasePlugin::NoVcsEnabled: {
+    case NoVcsEnabled: {
         const bool supportsCreation = d->supportsRepositoryCreation();
         menuAction->setVisible(supportsCreation);
         menuAction->setEnabled(supportsCreation);
         return supportsCreation;
     }
-    case VcsBase::VcsBasePlugin::OtherVcsEnabled:
+    case OtherVcsEnabled:
         menuAction->setVisible(false);
         return false;
-    case VcsBase::VcsBasePlugin::VcsEnabled:
+    case VcsEnabled:
         menuAction->setVisible(true);
         menuAction->setEnabled(true);
         break;
@@ -627,13 +637,24 @@ bool VcsBasePlugin::enableMenuAction(ActionState as, QAction *menuAction) const
     return true;
 }
 
+QString VcsBasePlugin::commitDisplayName() const
+{
+    return tr("Commit", "name of \"commit\" action of the VCS.");
+}
+
+bool VcsBasePlugin::promptBeforeCommit()
+{
+    return DocumentManager::saveAllModifiedDocuments(tr("Save before %1?")
+                                                     .arg(commitDisplayName().toLower()));
+}
+
 void VcsBasePlugin::promptToDeleteCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    const bool rc = Core::VcsManager::promptToDelete(versionControl(), state.currentFile());
+    const bool rc = VcsManager::promptToDelete(versionControl(), state.currentFile());
     if (!rc)
-        QMessageBox::warning(Core::ICore::dialogParent(), tr("Version Control"),
+        QMessageBox::warning(ICore::dialogParent(), tr("Version Control"),
                              tr("The file \"%1\" could not be deleted.").
                              arg(QDir::toNativeSeparators(state.currentFile())),
                              QMessageBox::Ok);
@@ -648,19 +669,19 @@ static inline bool ask(QWidget *parent, const QString &title, const QString &que
 
 void VcsBasePlugin::createRepository()
 {
-    QTC_ASSERT(d->m_versionControl->supportsOperation(Core::IVersionControl::CreateRepositoryOperation), return);
+    QTC_ASSERT(d->m_versionControl->supportsOperation(IVersionControl::CreateRepositoryOperation), return);
     // Find current starting directory
     QString directory;
-    if (const ProjectExplorer::Project *currentProject = ProjectExplorer::ProjectExplorerPlugin::currentProject())
-        directory = QFileInfo(currentProject->document()->filePath()).absolutePath();
+    if (const Project *currentProject = ProjectTree::currentProject())
+        directory = currentProject->projectFilePath().toString();
     // Prompt for a directory that is not under version control yet
-    QWidget *mw = Core::ICore::mainWindow();
+    QWidget *mw = ICore::mainWindow();
     do {
         directory = QFileDialog::getExistingDirectory(mw, tr("Choose Repository Directory"), directory);
         if (directory.isEmpty())
             return;
-        const Core::IVersionControl *managingControl = Core::VcsManager::findVersionControlForDirectory(directory);
-        if (managingControl == 0)
+        const IVersionControl *managingControl = VcsManager::findVersionControlForDirectory(directory);
+        if (managingControl == nullptr)
             break;
         const QString question = tr("The directory \"%1\" is already managed by a version control system (%2)."
                                     " Would you like to specify another directory?").arg(directory, managingControl->displayName());
@@ -696,7 +717,7 @@ bool VcsBasePlugin::raiseSubmitEditor() const
 {
     if (!d->m_submitEditor)
         return false;
-    Core::EditorManager::activateEditor(d->m_submitEditor, Core::EditorManager::IgnoreNavigationHistory);
+    EditorManager::activateEditor(d->m_submitEditor, EditorManager::IgnoreNavigationHistory);
     return true;
 }
 
@@ -710,8 +731,7 @@ bool VcsBasePlugin::raiseSubmitEditor() const
 QString VcsBasePlugin::findRepositoryForDirectory(const QString &dirS,
                                                   const QString &checkFile)
 {
-    if (debugRepositorySearch)
-        qDebug() << ">VcsBasePlugin::findRepositoryForDirectory" << dirS << checkFile;
+    qCDebug(findRepoLog) << ">" << dirS << checkFile;
     QTC_ASSERT(!dirS.isEmpty() && !checkFile.isEmpty(), return QString());
 
     const QString root = QDir::rootPath();
@@ -724,13 +744,11 @@ QString VcsBasePlugin::findRepositoryForDirectory(const QString &dirS,
             break;
 
         if (QFileInfo(directory, checkFile).isFile()) {
-            if (debugRepositorySearch)
-                qDebug() << "<VcsBasePlugin::findRepositoryForDirectory> " << absDirPath;
+            qCDebug(findRepoLog) << "<" << absDirPath;
             return absDirPath;
         }
     } while (!directory.isRoot() && directory.cdUp());
-    if (debugRepositorySearch)
-        qDebug() << "<VcsBasePlugin::findRepositoryForDirectory bailing out at " << directory.absolutePath();
+    qCDebug(findRepoLog) << "< bailing out at" << directory.absolutePath();
     return QString();
 }
 
@@ -747,13 +765,13 @@ bool VcsBasePlugin::isSshPromptConfigured()
 
 static const char SOURCE_PROPERTY[] = "qtcreator_source";
 
-void VcsBasePlugin::setSource(Core::IDocument *document, const QString &source)
+void VcsBasePlugin::setSource(IDocument *document, const QString &source)
 {
     document->setProperty(SOURCE_PROPERTY, source);
     VcsBasePluginPrivate::m_listener->slotStateChanged();
 }
 
-QString VcsBasePlugin::source(Core::IDocument *document)
+QString VcsBasePlugin::source(IDocument *document)
 {
     return document->property(SOURCE_PROPERTY).toString();
 }
@@ -762,27 +780,28 @@ void VcsBasePlugin::setProcessEnvironment(QProcessEnvironment *e,
                                           bool forceCLocale,
                                           const QString &sshPromptBinary)
 {
-    if (forceCLocale)
-        e->insert(QLatin1String("LANG"), QString(QLatin1Char('C')));
+    if (forceCLocale) {
+        e->insert("LANG", "C");
+        e->insert("LANGUAGE", "C");
+    }
     if (!sshPromptBinary.isEmpty())
-        e->insert(QLatin1String("SSH_ASKPASS"), sshPromptBinary);
+        e->insert("SSH_ASKPASS", sshPromptBinary);
 }
 
 // Run a process synchronously, returning Utils::SynchronousProcessResponse
 // response struct and using the VcsBasePlugin flags as applicable
 SynchronousProcessResponse VcsBasePlugin::runVcs(const QString &workingDir,
-                                                 const QString &binary,
+                                                 const FileName &binary,
                                                  const QStringList &arguments,
-                                                 int timeOutMS,
+                                                 int timeOutS,
                                                  unsigned flags,
                                                  QTextCodec *outputCodec,
                                                  const QProcessEnvironment &env)
 {
-    Command command(binary, workingDir,
-                    env.isEmpty() ? QProcessEnvironment::systemEnvironment() : env);
+    VcsCommand command(workingDir, env.isEmpty() ? QProcessEnvironment::systemEnvironment() : env);
     command.addFlags(flags);
     command.setCodec(outputCodec);
-    return command.runVcs(arguments, timeOutMS);
+    return command.runCommand(binary, arguments, timeOutS);
 }
 
 } // namespace VcsBase
